import os
from typing import Any, Dict

from dynaconf import Dynaconf


def post(settings: Dynaconf) -> Dict[str, Any]:
    """The dynaconf post hook is called after all the settings are loaded and set.

    Post hook is necessary when a setting key depends conditionally on a previouslys et variable.

    settings: A read-only copy of the django.conf.settings
    returns: a dictionary to be merged to django.conf.settings
    """
    data = {"dynaconf_merge": False}
    # existing keys will be merged if dynaconf_merge is set to True
    # here it is set to false, so it allows each value to be individually marked as a merge.

    data.update(configure_logging(settings))
    data.update(configure_keycloak(settings))
    data.update(configure_cors(settings))

    return data


def configure_keycloak(settings: Dynaconf) -> Dict[str, Any]:
    """Configure keycloak settings for galaxy.

    This function returns a dictionary that will be merged to the settings.
    """

    data = {}

    # Obtain values for Social Auth
    SOCIAL_AUTH_KEYCLOAK_KEY = settings.get("SOCIAL_AUTH_KEYCLOAK_KEY", default=None)
    SOCIAL_AUTH_KEYCLOAK_SECRET = settings.get("SOCIAL_AUTH_KEYCLOAK_SECRET", default=None)
    SOCIAL_AUTH_KEYCLOAK_PUBLIC_KEY = settings.get("SOCIAL_AUTH_KEYCLOAK_PUBLIC_KEY", default=None)
    KEYCLOAK_PROTOCOL = settings.get("KEYCLOAK_PROTOCOL", default=None)
    KEYCLOAK_HOST = settings.get("KEYCLOAK_HOST", default=None)
    KEYCLOAK_PORT = settings.get("KEYCLOAK_PORT", default=None)
    KEYCLOAK_REALM = settings.get("KEYCLOAK_REALM", default=None)

    # Add settings if Social Auth values are provided
    if all(
        [
            SOCIAL_AUTH_KEYCLOAK_KEY,
            SOCIAL_AUTH_KEYCLOAK_SECRET,
            SOCIAL_AUTH_KEYCLOAK_PUBLIC_KEY,
            KEYCLOAK_HOST,
            KEYCLOAK_PORT,
            KEYCLOAK_REALM,
        ]
    ):
        data["KEYCLOAK_ADMIN_ROLE"] = settings.get("KEYCLOAK_ADMIN_ROLE", default="hubadmin")
        data["KEYCLOAK_GROUP_TOKEN_CLAIM"] = settings.get(
            "KEYCLOAK_GROUP_TOKEN_CLAIM", default="group"
        )
        data["KEYCLOAK_ROLE_TOKEN_CLAIM"] = settings.get(
            "KEYCLOAK_GROUP_TOKEN_CLAIM", default="client_roles"
        )
        data["KEYCLOAK_HOST_LOOPBACK"] = settings.get("KEYCLOAK_HOST_LOOPBACK", default=None)
        data["KEYCLOAK_URL"] = f"{KEYCLOAK_PROTOCOL}://{KEYCLOAK_HOST}:{KEYCLOAK_PORT}"
        auth_url_str = "{keycloak}/auth/realms/{realm}/protocol/openid-connect/auth/"
        data["SOCIAL_AUTH_KEYCLOAK_AUTHORIZATION_URL"] = auth_url_str.format(
            keycloak=data["KEYCLOAK_URL"], realm=KEYCLOAK_REALM
        )
        if data["KEYCLOAK_HOST_LOOPBACK"]:
            loopback_url = "{protocol}://{host}:{port}".format(
                protocol=KEYCLOAK_PROTOCOL, host=data["KEYCLOAK_HOST_LOOPBACK"], port=KEYCLOAK_PORT
            )
            data["SOCIAL_AUTH_KEYCLOAK_AUTHORIZATION_URL"] = auth_url_str.format(
                keycloak=loopback_url, realm=KEYCLOAK_REALM
            )

        data[
            "SOCIAL_AUTH_KEYCLOAK_ACCESS_TOKEN_URL"
        ] = f"{data['KEYCLOAK_URL']}/auth/realms/{KEYCLOAK_REALM}/protocol/openid-connect/token/"

        data["SOCIAL_AUTH_LOGIN_REDIRECT_URL"] = settings.get(
            "SOCIAL_AUTH_LOGIN_REDIRECT_URL", default="/ui/"
        )
        data["SOCIAL_AUTH_POSTGRES_JSONFIELD"] = True
        data["SOCIAL_AUTH_JSONFIELD_CUSTOM"] = "django.contrib.postgres.fields.JSONField"
        data["SOCIAL_AUTH_URL_NAMESPACE"] = "social"
        data["SOCIAL_AUTH_KEYCLOAK_EXTRA_DATA"] = [
            ("refresh_token", "refresh_token"),
            (data["KEYCLOAK_ROLE_TOKEN_CLAIM"], data["KEYCLOAK_ROLE_TOKEN_CLAIM"]),
        ]

        data["SOCIAL_AUTH_PIPELINE"] = (
            "social_core.pipeline.social_auth.social_details",
            "social_core.pipeline.social_auth.social_uid",
            "social_core.pipeline.social_auth.social_user",
            "social_core.pipeline.user.get_username",
            "social_core.pipeline.social_auth.associate_by_email",
            "social_core.pipeline.user.create_user",
            "social_core.pipeline.social_auth.associate_user",
            "social_core.pipeline.social_auth.load_extra_data",
            "social_core.pipeline.user.user_details",
            "galaxy_ng.app.pipelines.user_role",
            "galaxy_ng.app.pipelines.user_group",
        )

        # Set external authentication feature flag
        # data["GALAXY_FEATURE_FLAGS"] = {'external_authentication': True, "dynaconf_merge": True}
        # The next have the same effect ^
        data["GALAXY_FEATURE_FLAGS__external_authentication"] = True

        # Add to installed apps
        data["INSTALLED_APPS"] = ["social_django", "dynaconf_merge"]

        # Add to authentication backends
        data["AUTHENTICATION_BACKENDS"] = [
            "social_core.backends.keycloak.KeycloakOAuth2",
            "dynaconf_merge",
        ]

        # Enable basic authentication using keycloak username and password
        data["REST_FRAMEWORK__DEFAULT_AUTHENTICATION_CLASSES"] = [
            "rest_framework.authentication.SessionAuthentication",
            "galaxy_ng.app.auth.keycloak.KeycloakBasicAuth",
        ]

        # Replace AUTH CLASSES
        data["GALAXY_AUTHENTICATION_CLASSES"] = [
            "rest_framework.authentication.SessionAuthentication",
            "galaxy_ng.app.auth.token.ExpiringTokenAuthentication",
            "galaxy_ng.app.auth.keycloak.KeycloakBasicAuth"
        ]

        # Set default to one day expiration
        data["GALAXY_TOKEN_EXPIRATION"] = settings.get("GALAXY_TOKEN_EXPIRATION", 1440)

        # Add to templates
        # Pending dynaconf issue:
        # https://github.com/rochacbruno/dynaconf/issues/299#issuecomment-900616706
        # So we can do a merge of this data.
        data["TEMPLATES"] = [
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "DIRS": [os.path.join(settings.BASE_DIR, "templates")],
                "APP_DIRS": True,
                "OPTIONS": {
                    "context_processors": [
                        # BEGIN: Pulp standard context processors
                        "django.template.context_processors.debug",
                        "django.template.context_processors.request",
                        "django.contrib.auth.context_processors.auth",
                        "django.contrib.messages.context_processors.messages",
                        # END: Pulp standard context processors
                        "social_django.context_processors.backends",
                        "social_django.context_processors.login_redirect",
                    ],
                },
            },
        ]

    return data


def configure_logging(settings: Dynaconf) -> Dict[str, Any]:
    data = {
        "GALAXY_ENABLE_API_ACCESS_LOG": settings.get(
            "GALAXY_ENABLE_API_ACCESS_LOG",
            default=os.getenv("GALAXY_ENABLE_API_ACCESS_LOG", default=False),
        )
    }
    if data["GALAXY_ENABLE_API_ACCESS_LOG"]:
        data["INSTALLED_APPS"] = ["automated_logging", "dynaconf_merge"]
        data["MIDDLEWARE"] = [
            "automated_logging.middleware.AutomatedLoggingMiddleware",
            "dynaconf_merge",
        ]
        data["LOGGING"] = {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "automated_logging": {"format": "%(asctime)s: %(levelname)s: %(message)s"},
            },
            "handlers": {
                "automated_logging": {
                    "level": "INFO",
                    "class": "logging.handlers.WatchedFileHandler",
                    "filename": "/var/log/galaxy_api_access.log",
                    "formatter": "automated_logging",
                },
            },
            "loggers": {
                "automated_logging": {
                    "handlers": ["automated_logging"],
                    "level": "INFO",
                    "propagate": False,
                },
            },
            "dynaconf_merge": True,
        }
        data["AUTOMATED_LOGGING"] = {
            "globals": {
                "exclude": {
                    "applications": [
                        "plain:contenttypes",
                        "plain:admin",
                        "plain:basehttp",
                        "glob:session*",
                        "plain:migrations",
                    ]
                }
            },
            "model": {
                "detailed_message": True,
                "exclude": {"applications": [], "fields": [], "models": [], "unknown": False},
                "loglevel": 20,
                "mask": [],
                "max_age": None,
                "performance": False,
                "snapshot": False,
                "user_mirror": False,
            },
            "modules": ["request", "unspecified", "model"],
            "request": {
                "data": {
                    "content_types": ["application/json"],
                    "enabled": [],
                    "ignore": [],
                    "mask": [
                        "ca_cert",
                        "client_cert",
                        "client_key",
                        "email",
                        "password",
                        "proxy_url",
                        "proxy_username",
                        "proxy_password",
                        "token",
                        "username",
                    ],
                    "query": True,
                },
                "exclude": {
                    "applications": [],
                    "methods": [],
                    "status": [],
                    "unknown": False,
                },
                "ip": True,
                "loglevel": 20,
                "max_age": None,
            },
            "unspecified": {
                "exclude": {"applications": [], "files": [], "unknown": False},
                "loglevel": 20,
                "max_age": None,
            },
        }

    return data


def configure_cors(settings: Dynaconf) -> Dict[str, Any]:
    """This adds CORS Middleware, useful to access swagger UI on dev environment"""

    if os.getenv("DEV_SOURCE_PATH", None) is None:
        # Only add CORS if we are in dev mode
        return {}

    data = {}
    if settings.get("GALAXY_ENABLE_CORS", default=False):
        corsmiddleware = ["galaxy_ng.app.common.openapi.AllowCorsMiddleware"]
        data["MIDDLEWARE"] = corsmiddleware + settings.get("MIDDLEWARE", [])
    return data



INSIGHTS_STATEMENTS = {
    'NamespaceViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "has_rh_entitlements",
        },
        {
            "action": "create",
            "principal": "authenticated",
            "effect": "allow",
            "condition": ["has_model_perms:galaxy.add_namespace", "has_rh_entitlements"]
        },
        {
            "action": "destroy",
            "principal": "authenticated",
            "effect": "allow",
            "condition": ["has_model_or_obj_perms:galaxy.delete_namespace", "has_rh_entitlements"]
        },
        {
            "action": "update",
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_or_obj_perms:galaxy.change_namespace",
                "has_rh_entitlements"]
        },
    ],
    'CollectionViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "has_rh_entitlements",
        },
        {
            "action": "destroy",
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:ansible.delete_collection",
                "has_rh_entitlements",
            ],
        },
        {
            "action": "create",
            "principal": "authenticated",
            "effect": "allow",
            "condition": ["can_create_collection", "has_rh_entitlements"]
        },
        {
            "action": "update",
            "principal": "authenticated",
            "effect": "allow",
            "condition": "can_update_collection"
        },
        {
            "action": "move_content",
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:ansible.modify_ansible_repo_content",
                "has_rh_entitlements"]
        },
        {
            "action": "curate",
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:ansible.modify_ansible_repo_content",
                "has_rh_entitlements"]
        }
    ],
    'CollectionRemoteViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow"
        },
        {
            "action": ["sync", "update", "partial_update"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "has_model_perms:ansible.change_collectionremote"
        }
    ],
    'UserViewSet': [
        {
            "action": ["*"],
            "principal": "authenticated",
            "effect": "deny",
        },
    ],
    'MyUserViewSet': [
        {
            "action": ["retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "is_current_user"
        },
    ],
    'SyncListViewSet': [
        {
            "action": ["list"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": ["has_model_perms:galaxy.view_synclist",
                          "has_rh_entitlements"]
        },
        {
            "action": ["retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_or_obj_perms:galaxy.view_synclist",
                "has_rh_entitlements"]
        },
        {
            "action": ["destroy"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:galaxy.delete_synclist",
                "has_rh_entitlements"]
        },
        {
            "action": ["create"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:galaxy.add_synclist",
                "has_rh_entitlements"]
        },
        {
            "action": ["update", "partial_update"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_model_perms:galaxy.change_synclist",
                "has_rh_entitlements"]
        },
    ],
    'MySyncListViewSet': [
        {
            "action": ["retrieve", "list", "update", "partial_update", "curate"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": [
                "has_rh_entitlements",
                "is_org_admin"]
        },
    ],
    'TaskViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
        },
    ],
    'TagViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
        },
    ],
    'GroupViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "has_rh_entitlements",
        },
    ],
    'DistributionViewSet': [
        {
            "action": ["*"],
            "principal": "authenticated",
            "effect": "deny",
        },
    ],
    'MyDistributionViewSet': [
        {
            "action": ["list", "retrieve"],
            "principal": "authenticated",
            "effect": "allow",
            "condition": "has_rh_entitlements",
        },
    ],
    'ContainerRepositoryViewSet': [
        {
            "action": ["*"],
            "principal": "*",
            "effect": "deny",
        },
    ],
}



import logging
import mimetypes

from django.conf import settings
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import extend_schema_field

from rest_framework import serializers
from rest_framework.exceptions import ValidationError, _get_error_details
from rest_framework.reverse import reverse

from galaxy_ng.app.api.ui.serializers.base import Serializer
from galaxy_ng.app.api.utils import parse_collection_filename

from pulp_ansible.app.galaxy.v3.serializers import (
    CollectionSerializer as _CollectionSerializer,
    CollectionRefSerializer as _CollectionRefSerializer,
    CollectionVersionSerializer as _CollectionVersionSerializer,
    UnpaginatedCollectionVersionSerializer as _UnpaginatedCollectionVersionSerializer,
    CollectionVersionListSerializer as _CollectionVersionListSerializer,
)

log = logging.getLogger(__name__)


class HrefNamespaceMixin:

    def _get_download_url(self, obj):
        """Get artifact download URL."""

        host = settings.ANSIBLE_API_HOSTNAME.strip("/")
        prefix = settings.GALAXY_API_PATH_PREFIX.strip("/")
        distro_base_path = self.context["path"]
        return f"{host}/{prefix}/v3/artifacts/collections/{distro_base_path}/{obj.relative_path}"

    def _get_href(self, url_name, **kwargs):
        """Generic get_*_href that uses context["view_namespace"] to reverse the right url"""

        view_namespace = self.context["view_namespace"]

        # Handle the case where the /api/automation-hub/v3/collections/ urls do not
        # include a "<str:path>" path param. ie, the bacwares compatible default "golden" repo
        if "/<str:path>/v3/" in self.context["view_route"]:
            kwargs.setdefault("path", self.context["path"])
        else:
            kwargs.pop("path", None)

        return reverse(
            f"{view_namespace}:{url_name}",
            kwargs=kwargs,
        )


class CollectionSerializer(_CollectionSerializer, HrefNamespaceMixin):
    class Meta(_CollectionSerializer.Meta):
        ref_name = "CollectionWithFixedHrefsSerializer"

    def get_href(self, obj) -> str:
        return self._get_href("collections-detail", namespace=obj.namespace, name=obj.name)

    def get_versions_url(self, obj) -> str:
        """Get a link to a collection versions list."""
        return self._get_href("collection-versions-list", namespace=obj.namespace, name=obj.name)

    @extend_schema_field(OpenApiTypes.OBJECT)
    def get_highest_version(self, obj):
        """Get a highest version and its link."""
        data = super().get_highest_version(obj)
        data['href'] = self._get_href(
            "collection-versions-detail",
            namespace=obj.namespace, name=obj.name, version=data['version']
        )
        return data


class CollectionRefSerializer(_CollectionRefSerializer, HrefNamespaceMixin):
    class Meta:
        ref_name = "CollectionWithFixedHrefsRefSerializer"

    def get_href(self, obj) -> str:
        """Returns link to a collection."""
        return self._get_href("collections-detail", namespace=obj.namespace, name=obj.name)


class CollectionVersionListSerializer(_CollectionVersionListSerializer, HrefNamespaceMixin):
    class Meta(_CollectionVersionListSerializer.Meta):
        ref_name = "CollectionVersionWithFixedHrefsRefListSerializer"

    def get_href(self, obj) -> str:
        """Get href."""
        return self._get_href(
            "collection-versions-detail",
            namespace=obj.namespace, name=obj.name, version=obj.version
        )


class UnpaginatedCollectionVersionSerializer(_UnpaginatedCollectionVersionSerializer,
                                             HrefNamespaceMixin):

    collection = CollectionRefSerializer(read_only=True)

    class Meta(_UnpaginatedCollectionVersionSerializer.Meta):
        ref_name = "UnpaginatedCollectionVersionWithFixedHrefsSerializer"

    def get_download_url(self, obj) -> str:
        return self._get_download_url(obj)

    def get_href(self, obj) -> str:
        return self._get_href("collections-detail", namespace=obj.namespace, name=obj.name)


class CollectionVersionSerializer(_CollectionVersionSerializer, HrefNamespaceMixin):

    collection = CollectionRefSerializer(read_only=True)

    class Meta(_CollectionVersionSerializer.Meta):
        ref_name = "CollectionVersionWithDownloadUrlSerializer"

    def get_download_url(self, obj) -> str:
        return self._get_download_url(obj)

    def get_href(self, obj) -> str:
        return self._get_href(
            "collection-versions-detail",
            namespace=obj.namespace, name=obj.name, version=obj.version
        )


class CollectionUploadSerializer(Serializer):
    """
    A serializer for the Collection One Shot Upload API.
    """

    file = serializers.FileField(required=True)

    sha256 = serializers.CharField(required=False, default=None)

    class Meta():
        ref_name = "CollectionUploadWithDownloadUrlSerializer"

    def to_internal_value(self, data):
        """Parse and validate collection filename."""
        data = super().to_internal_value(data)

        errors = {}

        filename = data["file"].name

        try:
            filename_tuple = parse_collection_filename(filename)
        except ValueError as exc:
            errors['filename'] = _get_error_details(exc, default_code='invalid')
            log.error('CollectionUploadSerializer validation of filename failed: %s',
                      errors['filename'])
            raise ValidationError(errors)

        data.update({
            "filename": filename_tuple,
            "expected_namespace": filename_tuple.namespace,
            "expected_name": filename_tuple.name,
            "expected_version": filename_tuple.version,
            "mimetype": (mimetypes.guess_type(filename)[0] or 'application/octet-stream')
        })
        return data



import os

MIDDLEWARE = [
    'django_prometheus.middleware.PrometheusBeforeMiddleware',
    # BEGIN: Pulp standard middleware
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    # END: Pulp standard middleware
    'django_prometheus.middleware.PrometheusAfterMiddleware',
    'django_currentuser.middleware.ThreadLocalUserMiddleware',
]

INSTALLED_APPS = [
    'rest_framework.authtoken',
    'dynaconf_merge',
]


AUTH_USER_MODEL = 'galaxy.User'

# FIXME(cutwater): 1. Rename GALAXY_API_ROOT in pulp-ansible to ANSIBLE_API_ROOT
#                  2. Rename API_PATH_PREFIX to GALAXY_API_ROOT
GALAXY_API_PATH_PREFIX = "/api/galaxy"
STATIC_URL = "/static/"
ADMIN_SITE_URL = "automation-hub/admin/"

# A client connection to /api/automation-hub/ is the same as a client connection
# to /api/automation-hub/content/<GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH>/
# with the exception of galaxy_ng.app.api.v3.viewsets.CollectionUploadViewSet
GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH = "published"

GALAXY_API_STAGING_DISTRIBUTION_BASE_PATH = "staging"
GALAXY_API_REJECTED_DISTRIBUTION_BASE_PATH = "rejected"

# The format for the name of the per account synclist, and the
# associated repository, distribution, and distribution base_paths
GALAXY_API_SYNCLIST_NAME_FORMAT = "{account_name}-synclist"

# Require approval for incoming content, which uses a staging repository
GALAXY_REQUIRE_CONTENT_APPROVAL = True

# Number of synclist to be processed in single task
SYNCLIST_BATCH_SIZE = 200

# Local rest framework settings
# -----------------------------

GALAXY_EXCEPTION_HANDLER = "galaxy_ng.app.api.exceptions.exception_handler"
GALAXY_PAGINATION_CLASS = "pulp_ansible.app.galaxy.v3.pagination.LimitOffsetPagination"
GALAXY_AUTHENTICATION_CLASSES = [
    "rest_framework.authentication.SessionAuthentication",
    "rest_framework.authentication.TokenAuthentication",
    "rest_framework.authentication.BasicAuthentication",
]
# Settings for insights mode
# GALAXY_AUTHENTICATION_CLASSES = ["galaxy_ng.app.auth.auth.RHIdentityAuthentication"]

# set to 'insights' for cloud.redhat.com deployments
GALAXY_DEPLOYMENT_MODE = 'standalone'

# Dictionary with True|False values for the application to turn on/off features
GALAXY_FEATURE_FLAGS = {
    'execution_environments': True,  # False will make execution_environments endpoints 404
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        'OPTIONS': {
            'min_length': 9,
        }
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Compatibility settings
# ----------------------
X_PULP_CONTENT_HOST = "localhost"
X_PULP_CONTENT_PORT = 24816

# Example setting of CONTENT_BIND if unix sockets are used
# CONTENT_BIND = "unix:/var/run/pulpcore-content/pulpcore-content.sock"

CONTENT_BIND = None

# This is example of how to set this via enviroment variables via dynaconf
# SPECTACULAR_SETTINGS__TITLE = "Automation Hub API __TITLE"

SPECTACULAR_SETTINGS = {
    "TITLE": "Automation Hub API",
    "DESCRIPTION": "Fetch, Upload, Organize, and Distribute Ansible Collections",
    "VERSION": "v3",
    "LICENSE": {
        "name": "GPLv2+",
        "url": "https://raw.githubusercontent.com/ansible/galaxy_ng/master/LICENSE",
    },
    "COMPONENT_SPLIT_REQUEST": True,
    "dynaconf_merge": True,
}

# Disable django guardian anonymous user
# https://django-guardian.readthedocs.io/en/stable/configuration.html#anonymous-user-name
ANONYMOUS_USER_NAME = None

# L10N settings
USE_L10N = True
USE_I18N = True

# Default language
LANGUAGE_CODE = 'en-us'

LANGUAGES = [
    # Django 3.0+ requires the language defined in LANGUAGE_CODE to be in this list
    ('en-us', 'English (USA)'),
    ('ja', 'Japanese'),
    ('nl', 'Dutch'),
    ('fr', 'French'),
    ('es', 'Spanish'),
    ('zh-hans', 'Chinese'),
]

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
LOCALE_PATHS = (os.path.join(BASE_DIR, 'locale'),)

CONNECTED_ANSIBLE_CONTROLLERS = []

GALAXY_ENABLE_UNAUTHENTICATED_COLLECTION_ACCESS = False
GALAXY_ENABLE_UNAUTHENTICATED_COLLECTION_DOWNLOAD = False

GALAXY_ENABLE_API_ACCESS_LOG = False
# Extra AUTOMATED_LOGGING settings are defined on dynaconf_hooks.py
# to be overridden by the /etc/pulp/settings.py
# or environment variable PULP_GALAXY_ENABLE_API_ACCESS_LOG

SOCIAL_AUTH_KEYCLOAK_KEY = None
SOCIAL_AUTH_KEYCLOAK_SECRET = None
SOCIAL_AUTH_KEYCLOAK_PUBLIC_KEY = None
KEYCLOAK_PROTOCOL = None
KEYCLOAK_HOST = None
KEYCLOAK_PORT = None
KEYCLOAK_REALM = None
# Extra KEYCLOAK settings are defined on dynaconf_hooks.py
# to be overridden by the /etc/pulp/settings.py
# or environment variable PULP_SOCIAL_AUTH_KEYCLOAK_KEY etc...

# This is used to enable or disable SSL validation on calls to the
# keycloak server. This setting has 3 options:
# False: SSL certificates are never verified.
# True: Valid SSL certificates are required. If this is set, the curl CA
#   bundle is used to validate server certs.
# /path/to/certfile: A custom CA certificate can be provided. If this is
#   set automation hub will use the CA certs in the file to validate
#   the keycloak's SSL certificate.
GALAXY_VERIFY_KEYCLOAK_SSL_CERTS = False



from requests import post as requests_post

from django.conf import settings

from rest_framework.authentication import BasicAuthentication

from rest_framework import status as http_code
from rest_framework import exceptions

from social_django.utils import load_strategy
from social_core.backends.keycloak import KeycloakOAuth2

from gettext import gettext as _


class KeycloakBasicAuth(BasicAuthentication):
    def authenticate_credentials(self, userid, password, request=None):
        payload = {
            'client_id': settings.SOCIAL_AUTH_KEYCLOAK_KEY,
            'client_secret': settings.SOCIAL_AUTH_KEYCLOAK_SECRET,
            'grant_type': 'password',
            'scope': 'openid',
            'username': userid,
            'password': password
        }
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
        }

        response = requests_post(
            url=settings.SOCIAL_AUTH_KEYCLOAK_ACCESS_TOKEN_URL,
            headers=headers,
            data=payload,
            verify=settings.GALAXY_VERIFY_KEYCLOAK_SSL_CERTS
        )

        if response.status_code == http_code.HTTP_200_OK:

            # load social auth django strategy
            strategy = load_strategy(request)
            backend = KeycloakOAuth2(strategy)

            token_data = backend.user_data(response.json()['access_token'])

            # The django social auth strategy uses data from the JWT token in the KeycloackOAuth2
            # backend to create a new user and update it with the data from the token. This
            # should return a django user instance.
            user = strategy.authenticate(backend, response=token_data)

            if user is None:
                raise exceptions.AuthenticationFailed(_("Authentication failed."))

            return (user, None)

        else:
            # If keycloak basic auth fails, try regular basic auth.
            return super().authenticate_credentials(userid, password, request)



from django.db import models
from pulpcore.plugin import models as pulp_models
from pulp_container.app import models as container_models
from django_lifecycle import LifecycleModelMixin

from galaxy_ng.app.access_control import mixins


class ContainerDistribution(
        container_models.ContainerDistribution,
        LifecycleModelMixin,
        mixins.GroupModelPermissionsMixin):

    class Meta:
        proxy = True
        default_related_name = "%(app_label)s_%(model_name)s"


class ContainerNamespace(
        container_models.ContainerNamespace,
        LifecycleModelMixin,
        mixins.GroupModelPermissionsMixin):

    class Meta:
        proxy = True
        default_related_name = "%(app_label)s_%(model_name)s"


class ContainerDistroReadme(models.Model):
    container = models.OneToOneField(
        container_models.ContainerDistribution,
        on_delete=models.CASCADE, primary_key=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True, null=True)
    text = models.TextField(blank=True)


class ContainerRegistryRemote(
    pulp_models.Remote,
    mixins.GroupModelPermissionsMixin,
):

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"

    RED_HAT_REGISTY_DOMAINS = (
        'registry.redhat.io',
    )

    def get_connection_fields(self):
        copy_fields = (
            "url",
            "policy",
            "username",
            "password",
            "tls_validation",
            "client_key",
            "client_cert",
            "ca_cert",
            "download_concurrency",
            "proxy_url",
            "proxy_username",
            "proxy_password",
            "rate_limit",
        )

        result = {}

        for field in copy_fields:
            result[field] = getattr(self, field)

        return result

    def get_registry_backend(self):
        for registry in self.RED_HAT_REGISTY_DOMAINS:
            if registry in self.url:
                return 'redhat'
        return None


class ContainerRegistryRepos(models.Model):
    registry = models.ForeignKey(
        ContainerRegistryRemote,
        on_delete=models.CASCADE,
    )
    repository_remote = models.OneToOneField(
        container_models.ContainerRemote,
        on_delete=models.CASCADE,
        primary_key=True,
        related_name="registry"
    )



# Generated by Django 2.2.15 on 2020-08-25 14:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0042_rbac_for_tasks'),
        ('ansible', '0021_rename_role_remote'),
        ('galaxy', '0004_rbac'),
    ]

    operations = [
        migrations.CreateModel(
            name='CollectionSyncTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ansible.AnsibleRepository')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Task')),
            ],
            options={
                'ordering': ['-task__finished_at'],
            },
        ),
    ]



import datetime

from django.conf import settings
from django.utils import timezone
from rest_framework.authentication import TokenAuthentication
from rest_framework.authtoken.models import Token
from rest_framework import exceptions


class ExpiringTokenAuthentication(TokenAuthentication):

    def authenticate_credentials(self, key):
        try:
            token = Token.objects.get(key=key)
        except Token.DoesNotExist:
            raise exceptions.AuthenticationFailed('Invalid token')

        if not token.user.is_active:
            raise exceptions.AuthenticationFailed('User inactive or deleted')

        # Token expiration only for SOCIAL AUTH users
        if hasattr(token.user, 'social_auth'):
            from social_django.models import UserSocialAuth
            try:
                token.user.social_auth.get(provider="keycloak")
                utc_now = timezone.now()
                # Set default to one day expiration
                try:
                    expiry = int(settings.get('GALAXY_TOKEN_EXPIRATION'))
                    if token.created < utc_now - datetime.timedelta(minutes=expiry):
                        raise exceptions.AuthenticationFailed('Token has expired')
                except ValueError:
                    pass
                except TypeError:
                    pass
            except UserSocialAuth.DoesNotExist:
                pass

        return (token.user, token)



from django.conf import settings
from django.conf.urls import url
from django.shortcuts import redirect
from django.urls import include, path

from . import views
from galaxy_ng.app.api import urls as api_urls
from galaxy_ng.app import customadmin as admin
from galaxy_ng.ui import urls as ui_urls

from drf_spectacular.views import (
    SpectacularJSONAPIView,
    SpectacularYAMLAPIView,
    SpectacularRedocView,
    SpectacularSwaggerView,
)

API_PATH_PREFIX = settings.GALAXY_API_PATH_PREFIX.strip("/")

galaxy_urls = [
    path(f"{API_PATH_PREFIX}/", include(api_urls)),
]

urlpatterns = [
    path("", include((galaxy_urls, "api"), namespace="galaxy")),
    path("", include(ui_urls)),
    path("", include("django_prometheus.urls")),
    path(settings.ADMIN_SITE_URL, admin.site.urls),
    path(
        f"{API_PATH_PREFIX}/v3/openapi.json",
        SpectacularJSONAPIView.as_view(),
        name="schema",
    ),
    path(
        f"{API_PATH_PREFIX}/v3/openapi.yaml",
        SpectacularYAMLAPIView.as_view(),
        name="schema-yaml",
    ),
    path(
        f"{API_PATH_PREFIX}/v3/redoc/",
        SpectacularRedocView.as_view(),
        name="schema-redoc",
    ),
    path(
        f"{API_PATH_PREFIX}/v3/swagger-ui/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger-ui",
    ),
    path("healthz", views.health_view),
]

if settings.get("SOCIAL_AUTH_KEYCLOAK_KEY"):
    urlpatterns.append(url("", include("social_django.urls", namespace="social")))
    urlpatterns.append(path("login/",
                       lambda request: redirect("/login/keycloak/", permanent=False)))



"""
Strictly speaking, signal handling and registration code can live anywhere you like, although it’s
recommended to avoid the application’s root module and its models module to minimize side-effects
of importing code.
In practice, signal handlers are usually defined in a signals submodule of the application they
relate to. Signal receivers are connected in the ready() method of your application configuration
class. If you’re using the receiver() decorator, simply import the signals submodule inside
ready().
https://stackoverflow.com/a/22924754
https://docs.djangoproject.com/en/3.2/topics/signals/
"""



from io import StringIO
from django.contrib.auth import get_user_model
from django.core.management import call_command, CommandError
from django.test import TestCase

User = get_user_model()


class TestCreateGroupCommand(TestCase):
    def setUp(self):
        super().setUp()
        userb = User.objects.create(username='userb')
        userb.save()

    def test_command_output(self):
        with self.assertRaisesMessage(
            CommandError, 'Error: the following arguments are required: groups'
        ):
            call_command('create-group')

    def test_add_group(self):
        out = StringIO()
        call_command('create-group', 'system:foo', stdout=out)
        self.assertIn("Created group 'system:foo'", out.getvalue())

    def test_group_already_exists(self):
        out = StringIO()
        call_command('create-group', 'system:foo', stdout=out)
        call_command('create-group', 'system:foo', stdout=out)
        self.assertIn("Group 'system:foo' already exists", out.getvalue())

    def test_add_group_and_assign(self):
        out = StringIO()
        call_command('create-group', 'system:foo', '--users', 'userb', stdout=out)
        self.assertIn("Assigned group 'system:foo' to user 'userb'", out.getvalue())

    def test_user_not_found(self):
        out = StringIO()
        call_command('create-group', 'system:foo', '--users', 'userz', stdout=out)
        self.assertIn("User 'userz' not found. Skipping", out.getvalue())



from django.db import transaction
from django.db.models import Q
from django.utils.translation import gettext_lazy as _
from django_filters import filters
from django_filters.rest_framework import DjangoFilterBackend, filterset
from pulp_ansible.app.models import Collection
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.request import Request
from rest_framework.response import Response

from galaxy_ng.app import models
from galaxy_ng.app.access_control.access_policy import NamespaceAccessPolicy
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.api.v3 import serializers
from galaxy_ng.app.exceptions import ConflictError
from galaxy_ng.app.models.namespace import delete_inbound_repo


class NamespaceFilter(filterset.FilterSet):
    keywords = filters.CharFilter(method='keywords_filter')

    sort = filters.OrderingFilter(
        fields=(
            ('name', 'name'),
            ('company', 'company'),
            ('id', 'id'),
        ),
    )

    class Meta:
        model = models.Namespace
        fields = ('name', 'company',)

    def keywords_filter(self, queryset, name, value):

        keywords = self.request.query_params.getlist('keywords')

        for keyword in keywords:
            queryset = queryset.filter(Q(name__icontains=keyword) | Q(company__icontains=keyword))

        return queryset


class NamespaceViewSet(api_base.ModelViewSet):
    lookup_field = "name"
    queryset = models.Namespace.objects.all()
    serializer_class = serializers.NamespaceSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = NamespaceFilter
    swagger_schema = None
    permission_classes = [NamespaceAccessPolicy]

    def get_serializer_class(self):
        if self.action == 'list':
            return serializers.NamespaceSummarySerializer
        else:
            return self.serializer_class

    @transaction.atomic
    def create(self, request: Request, *args, **kwargs) -> Response:
        """Override to validate for name duplication before serializer validation."""
        name = request.data.get('name')
        if name and models.Namespace.objects.filter(name=name).exists():
            # Ensures error raised is 409, not 400.
            raise ConflictError(
                detail={'name': _('A namespace named %s already exists.') % name}
            )
        return super().create(request, *args, **kwargs)

    @transaction.atomic
    def destroy(self, request: Request, *args, **kwargs) -> Response:
        """Delete a namespace.

        1. Perform a check to see if there are any collections in the namespace.
           If there are, return a failure.
        2. Delete the inbound pulp distro and repository
        3. Delete the namespace object.

        return: Response(status=204)
        """
        namespace = self.get_object()

        # 1. Check if there are any collections in the namespace.
        if Collection.objects.filter(namespace=namespace.name).exists():
            raise ValidationError(
                detail=_(
                    "Namespace {name} cannot be deleted because "
                    "there are still collections associated with it."
                ).format(name=namespace.name)
            )

        # 2. Delete the inbound pulp distro and repository
        #    the Namespace model delete will handle this but
        #    was kept here for better clarity.
        delete_inbound_repo(namespace.name)

        # 3. Delete the namespace object.
        self.perform_destroy(namespace)

        return Response(status=status.HTTP_204_NO_CONTENT)



from rest_framework import serializers


class Serializer(serializers.Serializer):
    def __init_subclass__(cls, **kwargs):
        """Set default attributes in subclasses.

        Sets the default for the ``ref_name`` attribute for a ModelSerializers's
        ``Meta`` class.

        If the ``Meta.ref_name`` attribute is not yet defined, set it according
        to the best practice established within Pulp: ``<app label>.<model class
        name>``. ``app_label`` is used to create a per plugin namespace.

        Serializers in pulpcore (``app_label`` is 'core') will not be
        namespaced, i.e. ref_name is not set in this case.

        The ``ref_name`` default value is computed using ``Meta.model``. If that
        is not defined (because the class must be subclassed to be useful),
        `ref_name` is not set.

        """
        super().__init_subclass__(**kwargs)
        meta = cls.Meta

        try:
            if not hasattr(meta, "ref_name"):
                meta.ref_name = f"galaxy.{cls.__name__.replace('Serializer', '')}"
        except AttributeError:
            pass

    class Meta:
        pass



import json
import re

from rest_framework import serializers
from django.core import exceptions
from django.db import transaction
from django.utils.translation import gettext_lazy as _
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import extend_schema_field

from guardian.shortcuts import get_users_with_perms

from pulp_container.app import models as container_models
from pulp_container.app import serializers as container_serializers
from pulpcore.plugin import models as core_models
from pulpcore.plugin import serializers as core_serializers

from galaxy_ng.app import models
from galaxy_ng.app.access_control.fields import GroupPermissionField, MyPermissionsField
from galaxy_ng.app.api import utils

namespace_fields = ("name", "my_permissions", "owners")

VALID_REMOTE_REGEX = r"^[A-Za-z0-9._-]*/?[A-Za-z0-9._-]*$"


class ContainerNamespaceSerializer(serializers.ModelSerializer):
    my_permissions = MyPermissionsField(source="*", read_only=True)
    owners = serializers.SerializerMethodField()

    class Meta:
        model = models.ContainerNamespace
        fields = namespace_fields
        read_only_fields = (
            "name",
            "my_permissions",
        )

    @extend_schema_field(serializers.ListField)
    def get_owners(self, namespace):
        return get_users_with_perms(namespace, with_group_users=False).values_list(
            "username", flat=True
        )


class ContainerNamespaceDetailSerializer(ContainerNamespaceSerializer):
    groups = GroupPermissionField()

    class Meta:
        model = models.ContainerNamespace
        fields = namespace_fields + ("groups",)
        read_only_fields = (
            "name",
            "my_permissions",
        )


class ContainerRepositorySerializer(serializers.ModelSerializer):
    pulp = serializers.SerializerMethodField()
    namespace = ContainerNamespaceSerializer()
    id = serializers.SerializerMethodField()
    created = serializers.SerializerMethodField()
    updated = serializers.SerializerMethodField()

    # This serializer is purposfully refraining from using pulp fields directly
    # in the top level response body. This is because future versions of hub will have to
    # support indexing other registries and the API responses for a container
    # repository should make sense for containers hosted by pulp and containers
    # hosted by other registries.
    class Meta:
        model = models.ContainerDistribution
        read_only_fields = (
            "id",
            "name",
            # this field will return null on instances where hub is indexing a
            # different repo
            "pulp",
            "namespace",
            "description",
            "created",
            "updated",
        )

        fields = read_only_fields

    def get_namespace(self, distro) -> str:
        return distro.namespace.name

    @extend_schema_field(OpenApiTypes.UUID)
    def get_id(self, distro):
        return distro.pk

    @extend_schema_field(OpenApiTypes.DATETIME)
    def get_created(self, distro):
        return distro.repository.pulp_created

    @extend_schema_field(OpenApiTypes.DATETIME)
    def get_updated(self, distro):
        return distro.repository.pulp_last_updated

    @extend_schema_field(OpenApiTypes.OBJECT)
    def get_pulp(self, distro):
        repo = distro.repository
        remote = None
        if repo.remote:
            remote = ContainerRemoteSerializer(repo.remote.cast()).data

        return {
            "repository": {
                "pulp_id": repo.pk,
                "pulp_type": repo.pulp_type,
                "version": repo.latest_version().number,
                "name": repo.name,
                "description": repo.description,
                "pulp_created": repo.pulp_created,
                "pulp_labels": {label.key: label.value for label in repo.pulp_labels.all()},
                "remote": remote
            },
            "distribution": {
                "pulp_id": distro.pk,
                "name": distro.name,
                "pulp_created": distro.pulp_created,
                "base_path": distro.base_path,
                "pulp_labels": {label.key: label.value for label in distro.pulp_labels.all()},
            },
        }


class ManifestListManifestSerializer(serializers.ModelSerializer):
    digest = serializers.SerializerMethodField()

    class Meta:
        model = container_models.ManifestListManifest
        fields = (
            "os",
            "architecture",
            "os_version",
            "os_features",
            "features",
            "variant",
            "digest"
        )

    def get_digest(self, obj):
        return obj.manifest_list.digest


class ContainerManifestSerializer(serializers.ModelSerializer):
    config_blob = serializers.SerializerMethodField()
    tags = serializers.SerializerMethodField()
    layers = serializers.SerializerMethodField()
    image_manifests = ManifestListManifestSerializer(many=True)

    class Meta:
        model = container_models.Manifest
        fields = (
            "pulp_id",
            "digest",
            "schema_version",
            "media_type",
            "config_blob",
            "tags",
            "pulp_created",
            "layers",
            "image_manifests"
        )

    @extend_schema_field(OpenApiTypes.OBJECT)
    def get_layers(self, obj):
        layers = []
        # use the prefetched blob_list and artifact_list instead of obj.blobs and
        # blob._artifacts to cut down on queries made.
        for blob in obj.blob_list:
            # Manifest can be empty after deleting an image.
            if blob.artifact_list:
                layers.append(
                    {"digest": blob.digest, "size": blob.artifact_list[0].size}
                )
        return layers

    @extend_schema_field(OpenApiTypes.OBJECT)
    def get_config_blob(self, obj):
        if not obj.config_blob:
            return {}
        return {"digest": obj.config_blob.digest, "media_type": obj.config_blob.media_type}

    @extend_schema_field(serializers.ListField)
    def get_tags(self, obj):
        tags = []
        # tagget_manifests returns all tags on the manifest, not just the ones
        # that are in the latest version of the repo.
        for tag in obj.tagged_manifests.all():
            tags.append(tag.name)

        return tags


class ManifestListSerializer(serializers.ModelSerializer):
    class Meta:
        model = container_models.Manifest
        fields = (
            "pulp_id",
            "digest",
            "schema_version",
            "media_type",
            "pulp_created",
        )


class ContainerTagSerializer(serializers.ModelSerializer):
    tagged_manifest = ManifestListSerializer()

    class Meta:
        model = container_models.Tag
        fields = (
            "name",
            "pulp_created",
            "pulp_last_updated",
            "tagged_manifest"
        )


class ContainerManifestDetailSerializer(ContainerManifestSerializer):
    @extend_schema_field(OpenApiTypes.OBJECT)
    def get_config_blob(self, obj):
        with obj.config_blob._artifacts.first().file.open() as f:
            config_json = json.load(f)

        return {
            "digest": obj.config_blob.digest,
            "media_type": obj.config_blob.media_type,
            "data": config_json,
        }


class ContainerRepositoryHistorySerializer(serializers.ModelSerializer):
    added = serializers.SerializerMethodField()
    removed = serializers.SerializerMethodField()

    class Meta:
        model = core_models.RepositoryVersion
        fields = ("pulp_id", "added", "removed", "pulp_created", "number")

    @extend_schema_field(serializers.ListField(child=serializers.JSONField()))
    def get_added(self, obj):
        return [self._content_info(content.content) for content in obj.added_memberships.all()]

    @extend_schema_field(serializers.ListField(child=serializers.JSONField()))
    def get_removed(self, obj):
        return [self._content_info(content.content) for content in obj.removed_memberships.all()]

    def _content_info(self, content):
        return_data = {
            "pulp_id": content.pk,
            "pulp_type": content.pulp_type,
            "manifest_digest": None,
            "tag_name": None,
        }

        # TODO: Figure out if there is a way to prefetch Manifest and Tag objects
        if content.pulp_type == "container.manifest":
            manifest = container_models.Manifest.objects.get(pk=content.pk)
            return_data["manifest_digest"] = manifest.digest
        elif content.pulp_type == "container.tag":
            tag = container_models.Tag.objects.select_related("tagged_manifest").get(pk=content.pk)
            return_data["manifest_digest"] = tag.tagged_manifest.digest
            return_data["tag_name"] = tag.name

        return return_data


class ContainerReadmeSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.ContainerDistroReadme
        fields = (
            "updated",
            "created",
            "text",
        )

        read_only_fields = (
            "updated",
            "created",
        )


class ContainerRegistryRemoteSerializer(
    core_serializers.RemoteSerializer,
):
    created_at = serializers.DateTimeField(source='pulp_created', required=False)
    updated_at = serializers.DateTimeField(source='pulp_last_updated', required=False)
    last_sync_task = utils.RemoteSyncTaskField(source="*")
    write_only_fields = serializers.SerializerMethodField()
    is_indexable = serializers.SerializerMethodField()

    class Meta:
        model = models.ContainerRegistryRemote
        fields = [
            "pk",
            "name",
            "url",
            "policy",
            "created_at",
            "updated_at",
            "username",
            "password",
            "tls_validation",
            "client_key",
            "client_cert",
            "ca_cert",
            "last_sync_task",
            "download_concurrency",
            "proxy_url",
            "proxy_username",
            "proxy_password",
            "write_only_fields",
            "rate_limit",
            "is_indexable"
        ]
        extra_kwargs = {
            'name': {'read_only': True},
            'client_key': {'write_only': True},
        }

    @extend_schema_field(serializers.ListField)
    def get_write_only_fields(self, obj):
        return utils.get_write_only_fields(self, obj)

    def get_is_indexable(self, obj) -> bool:
        if obj.get_registry_backend():
            return True
        return False


class ContainerRemoteSerializer(
    container_serializers.ContainerRemoteSerializer,
):
    created_at = serializers.DateTimeField(source='pulp_created', read_only=True, required=False)
    updated_at = serializers.DateTimeField(
        source='pulp_last_updated', read_only=True, required=False)
    registry = serializers.CharField(source="registry.registry.pk")
    last_sync_task = utils.RemoteSyncTaskField(source="*")

    class Meta:
        read_only_fields = (
            "created_at",
            "updated_at",
            "name"
        )

        model = container_models.ContainerRemote
        extra_kwargs = {
            'name': {'read_only': True},
            'client_key': {'write_only': True},
        }

        fields = [
            "pulp_id",
            "name",
            "upstream_name",
            "registry",
            "last_sync_task",
            "created_at",
            "updated_at",
            "include_foreign_layers",
            "include_tags",
            "exclude_tags"
        ]

    def validate_registry(self, value):
        try:
            registry = models.ContainerRegistryRemote.objects.get(pk=value)
            return registry
        except exceptions.ObjectDoesNotExist:
            raise serializers.ValidationError(_("Selected registry does not exist."))

    # pulp container doesn't validate container names and I don't know what is considered a
    # valid name.This is a stopgap solution to make sure that at the very least, users
    # don't create names that breakthe galaxy_ng registry
    def validate_name(self, value):
        r = re.compile(VALID_REMOTE_REGEX)
        if not r.match(value):
            raise serializers.ValidationError(
                _('Container names can only contain alphanumeric numbers, '
                    '".", "_", "-" and a up to one "/".'))
        return value

    @transaction.atomic
    def update(self, instance, validated_data):

        registry = validated_data['registry']['registry']['pk']
        del validated_data['registry']

        # Ensure connection exists between remote and registry
        models.container.ContainerRegistryRepos.objects.get_or_create(
            repository_remote=instance,
            defaults={'registry': registry, 'repository_remote': instance}
        )

        if (instance.name != validated_data['name']):
            raise serializers.ValidationError(detail={
                "name": _("Name cannot be changed.")
            })

        instance.registry.registry = registry
        instance.registry.save()
        validated_data = {**registry.get_connection_fields(), **validated_data}
        return super().update(instance, validated_data)

    @transaction.atomic
    def create(self, validated_data):
        registry = validated_data['registry']['registry']['pk']
        del validated_data['registry']

        validated_data = {**registry.get_connection_fields(), **validated_data}

        # Exclude source tags by default since they don't provide much value to customers and
        # can cause network issues when syncing.
        validated_data["exclude_tags"] = validated_data.get("exclude_tags", [])
        if "*-source" not in validated_data["exclude_tags"]:
            validated_data["exclude_tags"].append("*-source")

        request = self.context['request']

        # Create the remote instances using data from the registry

        remote = super().create(validated_data)
        remote_href = container_serializers.ContainerRemoteSerializer(
            remote, context={"request": request}).data['pulp_href']

        # Create the container repository with the new remote
        repo_serializer = container_serializers.ContainerRepositorySerializer(
            data={"name": remote.name, "remote": remote_href}, context={"request": request}
        )
        repo_serializer.is_valid(raise_exception=True)
        repository = repo_serializer.create(repo_serializer.validated_data)
        repo_href = container_serializers.ContainerRepositorySerializer(
            repository, context={"request": request}
        ).data["pulp_href"]

        # Create the container distribution with the new repository
        dist_serializer = container_serializers.ContainerDistributionSerializer(
            data={"base_path": remote.name, "name": remote.name, "repository": repo_href}
        )
        dist_serializer.is_valid(raise_exception=True)
        dist_serializer.create(dist_serializer.validated_data)

        # Bind the new remote to the registry object.
        models.ContainerRegistryRepos.objects.create(registry=registry, repository_remote=remote)

        return remote



# Generated by Django 2.2.18 on 2021-02-12 17:29

from django.db import migrations
import django_lifecycle.mixins
import galaxy_ng.app.access_control.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('container', '0018_containerdistribution_description'),
        ('galaxy', '0013_partner_engineer_group_migrations'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContainerDistribution',
            fields=[
            ],
            options={
                'proxy': True,
                'default_related_name': '%(app_label)s_%(model_name)s',
                'indexes': [],
                'constraints': [],
            },
            bases=('container.containerdistribution', django_lifecycle.mixins.LifecycleModelMixin, galaxy_ng.app.access_control.mixins.GroupModelPermissionsMixin),
        ),
    ]



from django.conf import settings
from django.urls import include, path, re_path
from rest_framework import routers

from galaxy_ng.app import constants

from . import views, viewsets

router = routers.SimpleRouter()
# TODO: Replace with a RedirectView
router.register('namespaces', viewsets.NamespaceViewSet, basename='namespaces')
router.register('my-namespaces', viewsets.MyNamespaceViewSet, basename='my-namespaces')
router.register('my-synclists', viewsets.MySyncListViewSet, basename='my-synclists')
router.register('users', viewsets.UserViewSet, basename='users')
router.register('collection-versions',
                viewsets.CollectionVersionViewSet, basename='collection-versions')
router.register(
    'imports/collections',
    viewsets.CollectionImportViewSet,
    basename='collection-imports',
)
router.register('tags', viewsets.TagsViewSet, basename='tags')
router.register('synclists', viewsets.SyncListViewSet, basename='synclists')
router.register('remotes', viewsets.CollectionRemoteViewSet, basename='remotes')
router.register('distributions', viewsets.DistributionViewSet, basename='distributions')
router.register('my-distributions', viewsets.MyDistributionViewSet, basename='my-distributions')

auth_views = [
    path("login/", views.LoginView.as_view(), name="auth-login"),
    path("logout/", views.LogoutView.as_view(), name="auth-logout"),
]

container_repo_paths = [
    path(
        'images/',
        viewsets.ContainerRepositoryManifestViewSet.as_view({'get': 'list'}),
        name='container-repository-images'),
    path(
        'images/<str:manifest_ref>/',
        viewsets.ContainerRepositoryManifestViewSet.as_view(
            {"get": "retrieve", "delete": "destroy"}
        ),
        name='container-repository-images-config-blob'),
    path(
        'history/',
        viewsets.ContainerRepositoryHistoryViewSet.as_view({'get': 'list'}),
        name='container-repository-history'),
    path(
        'readme/',
        viewsets.ContainerReadmeViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='container-repository-readme'),
    path(
        'tags/',
        viewsets.ContainerTagViewset.as_view({'get': 'list'}),
        name='container-repository-tags'),
    path(
        "sync/",
        views.ContainerSyncRemoteView.as_view(),
        name='container-repository-sync'),
]

container_paths = [
    path(
        "repositories/",
        viewsets.ContainerRepositoryViewSet.as_view({'get': 'list'}),
        name='container-repository-list'),
    path(
        "namespaces/<str:name>/",
        viewsets.ContainerNamespaceViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='container-namespace-detail'),
    path(
        "namespaces/",
        viewsets.ContainerNamespaceViewSet.as_view({'get': 'list'}),
        name='container-namespace-list'),
    path(
        "registries/<str:pk>/",
        viewsets.ContainerRegistryRemoteViewSet.as_view(
            {'get': 'retrieve', 'put': 'update', 'delete': 'destroy'}
        ),
        name='execution-environments-registry-detail'),
    path(
        "registries/<str:pk>/sync/",
        views.ContainerSyncRegistryView.as_view(),
        name='container-registry-sync'),
    path(
        "registries/<str:pk>/index/",
        views.IndexRegistryEEView.as_view(),
        name='execution-environments-registry-index'),
    path(
        "registries/",
        viewsets.ContainerRegistryRemoteViewSet.as_view({'get': 'list', 'post': 'create'}),
        name='execution-environments-registry-list'),
    path(
        "remotes/<str:pk>/",
        viewsets.ContainerRemoteViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='execution-environments-remote-detail'),
    path(
        "remotes/",
        viewsets.ContainerRemoteViewSet.as_view({'get': 'list', 'post': 'create'}),
        name='execution-environments-remote-list'),

    # image names can't start with _, so namespacing all the nested views
    # under _content prevents cases where an image could be named foo/images
    # and conflict with our URL paths.
    re_path(
        r'repositories/(?P<base_path>[-\w.]+\/{0,1}[-\w.]+)/_content/',
        include(container_repo_paths)),

    # This regex can capture "namespace/name" and "name"
    re_path(
        r"repositories/(?P<base_path>[-\w.]+\/{0,1}[-\w.]+)/",
        viewsets.ContainerRepositoryViewSet.as_view({"get": "retrieve", "delete": "destroy"}),
        name="container-repository-detail",
    ),
]

# Groups are subclassed from pulpcore and use nested viewsets, so router.register
# unfortunately doesn't work
group_paths = [
    path(
        "",
        viewsets.GroupViewSet.as_view({'get': 'list', 'post': 'create'}),
        name='groups'),
    path(
        "<str:pk>/",
        viewsets.GroupViewSet.as_view({'get': 'retrieve', 'delete': 'destroy'}),
        name='group-detail'),
    path(
        "<str:group_pk>/model-permissions/",
        viewsets.GroupModelPermissionViewSet.as_view({
            'get': 'list', 'post': 'create'}),
        name='group-model-permissions'),
    path(
        "<str:group_pk>/model-permissions/<str:pk>/",
        viewsets.GroupModelPermissionViewSet.as_view({
            'get': 'retrieve', 'delete': 'destroy'}),
        name='group-model-permissions-detail'),
    path(
        "<str:group_pk>/users/",
        viewsets.GroupUserViewSet.as_view({
            'get': 'list', 'post': 'create'}),
        name='group-model-permissions'),
    path(
        "<str:group_pk>/users/<str:pk>/",
        viewsets.GroupUserViewSet.as_view({
            'delete': 'destroy'}),
        name='group-model-permissions-detail'),
]

paths = [
    path('', include(router.urls)),

    path('auth/', include(auth_views)),
    path("settings/", views.SettingsView.as_view(), name="settings"),
    path('feature-flags/', views.FeatureFlagsView.as_view(), name='feature-flags'),
    path('controllers/', views.ControllerListView.as_view(), name='controllers'),
    path('groups/', include(group_paths)),
    path(
        'repo/<str:path>/',
        viewsets.CollectionViewSet.as_view({'get': 'list'}),
        name='collections-list'
    ),
    path(
        'repo/<str:path>/<str:namespace>/<str:name>/',
        viewsets.CollectionViewSet.as_view({'get': 'retrieve'}),
        name='collections-detail'
    ),
    # NOTE: Using path instead of SimpleRouter because SimpleRouter expects retrieve
    # to look up values with an ID
    path(
        'me/',
        viewsets.CurrentUserViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='me'
    ),
]

if settings.GALAXY_FEATURE_FLAGS['execution_environments']:
    paths.append(
        path('execution-environments/', include(container_paths)),
    )

app_name = "ui"

urlpatterns = [
    path('', viewsets.APIRootView.as_view({'get': 'list'}))
]

for version in constants.ALL_UI_API_VERSION:
    urlpatterns.append(path(
        constants.ALL_UI_API_VERSION[version],
        include((paths, app_name), namespace=version)
    ))



from galaxy_ng.app.constants import INBOUND_REPO_NAME_FORMAT
from django.test import TestCase
from pulp_ansible.app.models import AnsibleDistribution, AnsibleRepository, Collection

from galaxy_ng.app.models import Namespace


class TestSignalCreateRepository(TestCase):
    def test_create_repository_ensure_retain_repo_versions(self):
        """On creation retain_repo_versions is set to 1 if omited"""
        repo_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name="test")
        repository = AnsibleRepository.objects.create(name=repo_name)
        self.assertEqual(repository.name, repo_name)
        self.assertEqual(repository.retain_repo_versions, 1)

    def test_when_set_not_changed_retain_repo_versions(self):
        """On creation retain_repo_versions is not changed when explicit set"""
        repo_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name="test2")
        repository = AnsibleRepository.objects.create(name=repo_name, retain_repo_versions=99)
        self.assertEqual(repository.name, repo_name)
        self.assertEqual(repository.retain_repo_versions, 99)

    def test_update_do_not_change_retain_repo_versions(self):
        """On update retain_repo_versions is not changed when already exists"""
        # Create repo setting retain_repo_versions
        repo_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name="test3")
        repository = AnsibleRepository.objects.create(name=repo_name, retain_repo_versions=99)
        self.assertEqual(repository.name, repo_name)
        self.assertEqual(repository.retain_repo_versions, 99)
        # Update the name of the repo
        AnsibleRepository.objects.filter(pk=repository.pk).update(name="test3_2")
        updated = AnsibleRepository.objects.get(pk=repository.pk)
        # Ensure name changed but retain_repo_versions did not
        self.assertEqual(updated.name, "test3_2")
        self.assertEqual(updated.retain_repo_versions, 99)


class TestSignalCreateNamespace(TestCase):
    namespace_name = 'my_test_ns'

    def test_new_collection_create_namespace(self):
        self.assertFalse(Namespace.objects.filter(name=self.namespace_name))
        Collection.objects.create(
            name='my_collection',
            namespace=self.namespace_name,
        )
        self.assertTrue(Namespace.objects.filter(name=self.namespace_name))

    def test_existing_namespace_not_changed(self):
        description = 'Namespace created not by signal'
        Namespace.objects.create(
            name=self.namespace_name,
            description=description,
        )
        Collection.objects.create(
            name='my_collection',
            namespace=self.namespace_name,
        )
        namespace = Namespace.objects.get(name=self.namespace_name)
        self.assertEquals(namespace.description, description)


class TestNamespaceModelManager(TestCase):
    namespace_name = 'my_test_ns_2'

    def test_new_namespace_creates_inbound_repo(self):
        """When creating a new Namespace the manager should create inbound instances."""
        self.assertFalse(Namespace.objects.filter(name=self.namespace_name))
        Namespace.objects.create(name=self.namespace_name)
        self.assertTrue(Namespace.objects.filter(name=self.namespace_name).exists())
        inbound_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name=self.namespace_name)
        self.assertTrue(AnsibleRepository.objects.filter(name=inbound_name).exists())
        self.assertTrue(AnsibleDistribution.objects.filter(name=inbound_name).exists())

    def test_delete_namespace_deletes_inbound_repo(self):
        """When deleting a Namespace the manager should delete inbound instances."""
        Namespace.objects.get_or_create(name=self.namespace_name)
        ns = Namespace.objects.get(name=self.namespace_name)
        ns.delete()

        inbound_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name=self.namespace_name)

        self.assertFalse(Namespace.objects.filter(name=self.namespace_name).exists())

        self.assertFalse(AnsibleRepository.objects.filter(name=inbound_name).exists())
        self.assertFalse(AnsibleDistribution.objects.filter(name=inbound_name).exists())



from django.contrib.auth import get_user_model
from django.core.management import BaseCommand

User = get_user_model()


class Command(BaseCommand):
    """
    Django management command for deleting a user
    """

    help = 'Delete a user'

    def add_arguments(self, parser):
        parser.add_argument('--user', metavar='USER', dest='username',
                            help='user name', required=True)

    def handle(self, *args, **options):
        username = options['username']

        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            self.stdout.write("User not found")
        else:
            user.delete()
            self.stdout.write("User '{}' deleted".format(username))



from django.conf import settings
from rest_framework.permissions import IsAuthenticatedOrReadOnly
from rest_framework.response import Response

from galaxy_ng.app.api import base as api_base


class SettingsView(api_base.APIView):
    permission_classes = [IsAuthenticatedOrReadOnly]

    def get(self, request, *args, **kwargs):
        data = {}
        keyset = [
            "GALAXY_ENABLE_UNAUTHENTICATED_COLLECTION_ACCESS",
            "GALAXY_ENABLE_UNAUTHENTICATED_COLLECTION_DOWNLOAD",
            "GALAXY_FEATURE_FLAGS",
            "GALAXY_TOKEN_EXPIRATION",
        ]
        data = {key: settings.as_dict().get(key, None) for key in keyset}
        return Response(data)



import base64
import json
import logging

from django.db import transaction
from django.conf import settings

from guardian import shortcuts

from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import AuthenticationFailed

from pulp_ansible.app.models import AnsibleDistribution, AnsibleRepository
from galaxy_ng.app.models import SyncList
from galaxy_ng.app.models.auth import Group, User


DEFAULT_UPSTREAM_REPO_NAME = settings.GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH
RH_ACCOUNT_SCOPE = 'rh-identity-account'
SYNCLIST_DEFAULT_POLICY = 'exclude'


log = logging.getLogger(__name__)


class RHIdentityAuthentication(BaseAuthentication):
    """
    Authenticates users based on RedHat identity header.

    For users logging in first time creates User record and
    Tenant record for user's account if it doesn't exist.
    """

    header = 'HTTP_X_RH_IDENTITY'

    def authenticate(self, request):
        """
        Authenticates user.

        Raises:
            AuthenticationFailed: If invalid identity header provided.
        """
        if self.header not in request.META:
            return None

        header = self._decode_header(request.META[self.header])

        try:
            identity = header['identity']
            account = identity['account_number']

            user = identity['user']
            username = user['username']
        except KeyError:
            raise AuthenticationFailed

        email = user.get('email', '')
        first_name = user.get('first_name', '')
        last_name = user.get('last_name', '')

        group, _ = self._ensure_group(RH_ACCOUNT_SCOPE, account)

        user = self._ensure_user(
            username,
            group,
            email=email,
            first_name=first_name,
            last_name=last_name
        )

        self._ensure_synclists(group)

        return user, {'rh_identity': header}

    def _ensure_group(self, account_scope, account):
        """Create a auto group for the account and create a synclist distribution"""

        with transaction.atomic():
            group, created = Group.objects.get_or_create_identity(account_scope, account)
        return group, created

    def _ensure_synclists(self, group):
        with transaction.atomic():
            # check for existing synclists
            perms = ['galaxy.view_synclist']

            synclists_owned_by_group = \
                shortcuts.get_objects_for_group(group, perms, klass=SyncList,
                                                any_perm=False, accept_global_perms=True)
            if synclists_owned_by_group:
                return synclists_owned_by_group

            upstream_repository = self._get_upstream_repo(group, DEFAULT_UPSTREAM_REPO_NAME)

            distro_name = settings.GALAXY_API_SYNCLIST_NAME_FORMAT.format(
                account_name=group.account_number()
            )

            repository = self._ensure_repository(distro_name)
            self._get_or_create_synclist_distribution(distro_name, upstream_repository)

            default_synclist = SyncList.objects.create(
                repository=repository, upstream_repository=upstream_repository,
                policy=SYNCLIST_DEFAULT_POLICY,
                name=distro_name)

            default_synclist.groups = {group: ['galaxy.view_synclist', 'galaxy.add_synclist',
                                               'galaxy.delete_synclist', 'galaxy.change_synclist']}
            default_synclist.save()
        return default_synclist

    @staticmethod
    def _ensure_repository(name):
        repository, created = AnsibleRepository.objects.get_or_create(name=name)
        if created:
            repository.save()
        return repository

    @staticmethod
    def _ensure_user(username, group, **attrs):
        with transaction.atomic():
            user, created = User.objects.update_or_create(
                username=username,
                defaults=attrs,
            )
            if created:
                user.groups.add(group)
        return user

    @staticmethod
    def _get_upstream_repo(group, repository_name):
        try:
            upstream_repo = AnsibleRepository.objects.get(name=repository_name)
        except AnsibleRepository.DoesNotExist as exc:
            log.exception(exc)
            raise

        return upstream_repo

    @staticmethod
    def _get_or_create_synclist_distribution(distro_name, upstream_repository):
        # Create a distro pointing to the upstream repository by default.
        # This distro will be updated to point to a synclist repo when the synclist
        # is populated.
        distro, _ = AnsibleDistribution.objects.get_or_create(
            name=distro_name, base_path=distro_name, repository=upstream_repository
        )
        return distro

    @staticmethod
    def _decode_header(raw):
        try:
            json_string = base64.b64decode(raw)
            return json.loads(json_string)
        except ValueError:
            raise AuthenticationFailed



from .auth import (
    LoginSerializer,
)
from .collection import (
    CollectionDetailSerializer,
    CollectionListSerializer,
    CollectionVersionSerializer,
    CollectionVersionDetailSerializer,
    CollectionVersionBaseSerializer,
)
from .imports import (
    ImportTaskDetailSerializer,
    ImportTaskListSerializer,
)

from .user import (
    UserSerializer,
    CurrentUserSerializer,
)

from .synclist import (
    SyncListSerializer,
    SyncListCollectionSummarySerializer,
)

from .distribution import (
    DistributionSerializer
)

from .execution_environment import (
    ContainerRepositorySerializer,
    ContainerManifestSerializer,
    ContainerRepositoryHistorySerializer,
    ContainerReadmeSerializer,
    ContainerManifestDetailSerializer,
    ContainerNamespaceDetailSerializer,
    ContainerRegistryRemoteSerializer,
    ContainerTagSerializer,
    ContainerRemoteSerializer,
)

__all__ = (
    # auth
    'LoginSerializer',
    # collection
    'CollectionDetailSerializer',
    'CollectionListSerializer',
    'CollectionVersionSerializer',
    'CollectionVersionDetailSerializer',
    'CollectionVersionBaseSerializer',
    # imports
    'ImportTaskDetailSerializer',
    'ImportTaskListSerializer',
    # current_user
    'CurrentUserSerializer',
    # user
    'UserSerializer',
    # synclist
    'SyncListSerializer',
    'SyncListCollectionSummarySerializer',
    # distribution
    'DistributionSerializer',
    # container
    'ContainerRepositorySerializer',
    'ContainerRepositoryImageSerializer',
    'ContainerRepositoryHistorySerializer',
    'ContainerManifestSerializer',
    'ContainerManifestDetailSerializer',
    'ContainerReadmeSerializer',
    'ContainerNamespaceDetailSerializer',
    'ContainerRegistryRemoteSerializer',
    'ContainerRemoteSerializer',
    'ContainerTagSerializer'
)



from django.conf import settings

import boto3
import logstash_formatter
import watchtower


AWS_ACCESS_KEY_ID = settings.CLOUDWATCH_ACCESS_KEY_ID
AWS_SECRET_ACCESS_KEY = settings.CLOUDWATCH_SECRET_ACCESS_KEY
AWS_REGION_NAME = settings.CLOUDWATCH_REGION_NAME

LOGGING_GROUP = settings.CLOUDWATCH_LOGGING_GROUP
LOGGING_STREAM_NAME = settings.CLOUDWATCH_LOGGING_STREAM_NAME


class CloudWatchHandler(watchtower.CloudWatchLogHandler):
    """Pre-configured CloudWatch handler."""

    def __init__(self):
        boto3_session = boto3.Session(
            aws_access_key_id=AWS_ACCESS_KEY_ID,
            aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
            region_name=AWS_REGION_NAME,
        )
        super().__init__(
            boto3_session=boto3_session,
            log_group=LOGGING_GROUP,
            stream_name=LOGGING_STREAM_NAME,
        )
        # TODO(cutwater): Request ID should not depend on Django framework
        # self.addFilter(request_id.logging.RequestIdFilter())
        self.setFormatter(logstash_formatter.LogstashFormatterV1())



from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError
from rest_framework import status

from drf_spectacular.utils import extend_schema

from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _

from pulpcore.plugin.tasking import dispatch
from pulpcore.plugin.viewsets import OperationPostponedResponse
from pulpcore.plugin.serializers import AsyncOperationResponseSerializer

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app import models
from galaxy_ng.app import tasks


class IndexRegistryEEView(api_base.APIView):
    permission_classes = [access_policy.ContainerRegistryRemoteAccessPolicy]
    action = 'index_execution_environments'

    @extend_schema(
        description="Trigger an asynchronous indexing task",
        responses={status.HTTP_202_ACCEPTED: AsyncOperationResponseSerializer},
    )
    def post(self, request: Request, *args, **kwargs) -> Response:
        registry = get_object_or_404(models.ContainerRegistryRemote, pk=kwargs['pk'])

        if not registry.get_registry_backend():
            raise ValidationError(
                detail={
                    'url':
                        _('Indexing execution environments is not supported on this registry.')
                })

        serializable_meta = {}
        for key in self.request.META:
            val = self.request.META[key]
            if type(val) == str:
                serializable_meta[key] = val

        request_data = {
            "META": serializable_meta,
        }

        result = dispatch(
            tasks.index_execution_environments_from_redhat_registry,
            kwargs={
                "registry_pk": registry.pk,
                "request_data": request_data
            },
        )

        return OperationPostponedResponse(result, self.request)



from .deletion import delete_collection, delete_collection_version  # noqa: F401
from .index_registry import index_execution_environments_from_redhat_registry  # noqa: F401
from .promotion import call_move_content_task  # noqa: F401
from .publishing import import_and_auto_approve, import_and_move_to_staging  # noqa: F401
from .registry_sync import launch_container_remote_sync, sync_all_repos_in_registry  # noqa: F401
from .synclist import curate_all_synclist_repository, curate_synclist_repository  # noqa: F401

# from .synchronizing import synchronize  # noqa



from django.test import override_settings

from rest_framework import status as http_code
from rest_framework.response import Response
from rest_framework.test import APIClient, APITestCase

from galaxy_ng.app.constants import DeploymentMode
from galaxy_ng.app.models import auth as auth_models

from .base import get_current_ui_url


@override_settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value)
class TestLoginViewsStandalone(APITestCase):
    def setUp(self):
        super().setUp()
        self.client = APIClient()

        self.login_url = get_current_ui_url("auth-login")
        self.logout_url = get_current_ui_url("auth-logout")
        self.me_url = get_current_ui_url("me")

        self.users = [
            auth_models.User.objects.create_user(username="test1", password="test1-secret"),
            auth_models.User.objects.create_user(username="test2", password="test2-secret"),
        ]

    def test_login(self):
        self._test_login("test1", "test1-secret")
        self._test_login("test2", "test2-secret")

    def test_login_csrf_cookie_set_header_missing(self):
        client = APIClient(enforce_csrf_checks=True)

        # Ensure CSRF cookie is set
        response = client.get(self.login_url)
        self.assertTrue("csrftoken" in response.cookies)

        self._test_login_permission_denied(
            data={"username": "test1", "password": "test1-secret"},
            expected_detail="CSRF Failed: CSRF token missing or incorrect.",
            client=client
        )

    def test_login_csrf_cookie_missing(self):
        client = APIClient(enforce_csrf_checks=True)
        self._test_login_permission_denied(
            data={"username": "test1", "password": "test1-secret"},
            expected_detail="CSRF Failed: CSRF cookie not set.",
            client=client
        )

    def test_login_invalid_password(self):
        response: Response = self.client.post(
            self.login_url, data={"username": "test1", "password": "invalid", }
        )
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)

        response: Response = self.client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)

    def test_login_wrong_password(self):
        response: Response = self.client.post(
            self.login_url, data={"username": "test2", "password": "test1-secret", }
        )
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)

    def test_login_no_username(self):
        data = {"password": "test1-secret"}
        self._test_login_validation_error(
            data,
            expected_code="required",
            expected_detail="This field is required.",
            expected_parameter="username",
        )

    def test_login_no_password(self):
        data = {"username": "test1"}
        self._test_login_validation_error(
            data,
            expected_code="required",
            expected_detail="This field is required.",
            expected_parameter="password",
        )

    def test_login_empty_username(self):
        data = {"username": "", "password": "test1-secret"}
        self._test_login_validation_error(
            data,
            expected_code="blank",
            expected_detail="This field may not be blank.",
            expected_parameter="username",
        )

    def test_login_empty_password(self):
        data = {"username": "test1", "password": ""}
        self._test_login_validation_error(
            data,
            expected_code="blank",
            expected_detail="This field may not be blank.",
            expected_parameter="password",
        )

    def test_logout(self):
        response: Response = self.client.post(
            self.login_url, data={"username": "test1", "password": "test1-secret", }
        )
        self.assertEqual(response.status_code, http_code.HTTP_204_NO_CONTENT)

        response: Response = self.client.post(self.logout_url)
        self.assertEqual(response.status_code, http_code.HTTP_204_NO_CONTENT)

        response: Response = self.client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)

    def _test_login(self, username, password, client=None):
        client = client or self.client
        response: Response = client.post(
            self.login_url, data={"username": username, "password": password}
        )
        self.assertEqual(response.status_code, http_code.HTTP_204_NO_CONTENT)
        self.assertTrue("csrftoken" in response.cookies)
        self.assertTrue("sessionid" in response.cookies)

        response: Response = client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_200_OK)
        self.assertEqual(response.data["username"], username)

    def _test_login_permission_denied(self, data, expected_detail, client=None):
        client = client or self.client
        response: Response = client.post(self.login_url, data)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)
        self.assertEqual(response.data, {
            "errors": [
                {
                    "status": "403",
                    "code": "permission_denied",
                    "detail": expected_detail,
                    "title": "You do not have permission to perform this action.",
                }
            ]
        })

    def _test_login_validation_error(
            self, data, expected_code, expected_detail, expected_parameter
    ):
        response: Response = self.client.post(self.login_url, data=data)

        self.assertEqual(response.status_code, http_code.HTTP_400_BAD_REQUEST)
        expected_response_data = {
            "errors": [
                {
                    "status": "400",
                    "code": expected_code,
                    "title": "Invalid input.",
                    "detail": expected_detail,
                    "source": {"parameter": expected_parameter},
                }
            ]
        }
        self.assertDictEqual(response.data, expected_response_data)



from io import StringIO
from django.contrib.auth import get_user_model
from django.core.management import call_command, CommandError
from django.test import TestCase

User = get_user_model()


class TestDeleteUserCommand(TestCase):
    def setUp(self):
        super().setUp()

    def test_command_output(self):
        with self.assertRaisesMessage(
            CommandError, 'Error: the following arguments are required: --user'
        ):
            call_command('delete-user')

    def test_delete_user(self):
        out = StringIO()
        username = 'system:foo'
        user = User.objects.create(username=username)
        user.save()
        call_command('delete-user', '--user', username, stdout=out)
        self.assertIn("User '{}' deleted".format(username), out.getvalue())

    def test_user_doesnt_exists(self):
        out = StringIO()
        call_command('delete-user', '--user', 'system:bar', stdout=out)
        self.assertIn("User not found", out.getvalue())



import logging

from django.core import exceptions
from django.db.models import Count, Prefetch, Q
from django.shortcuts import get_object_or_404
from django_filters import filters
from django_filters.rest_framework import DjangoFilterBackend, filterset
from drf_spectacular.utils import extend_schema
from guardian.shortcuts import get_objects_for_user
from pulp_container.app import models as container_models
from pulpcore.plugin import models as core_models
from pulpcore.plugin.serializers import AsyncOperationResponseSerializer
from pulpcore.plugin.tasking import dispatch
from pulpcore.plugin.viewsets import OperationPostponedResponse

from galaxy_ng.app import models
from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.api.ui import serializers
from galaxy_ng.app.tasks.deletion import (
    delete_container_distribution,
    delete_container_image_manifest,
)

log = logging.getLogger(__name__)


class RepositoryFilter(filterset.FilterSet):
    my_permissions = filters.CharFilter(method='has_permissions')

    sort = filters.OrderingFilter(
        fields=(
            ('repository__pulp_created', 'created'),
            ('name', 'name'),
            ('description', 'description'),
            ('repository__pulp_last_updated', 'updated'),
        ),
    )

    class Meta:
        model = models.ContainerDistribution
        fields = {
            'name': ['exact', 'icontains', 'contains', 'startswith'],
            'description': ['exact', 'icontains', 'contains', 'startswith'],
        }

    def has_permissions(self, queryset, name, value):
        perms = self.request.query_params.getlist(name)
        namespaces = get_objects_for_user(
            self.request.user, perms, klass=container_models.ContainerNamespace)
        return self.queryset.filter(namespace__in=namespaces)


class ManifestFilter(filterset.FilterSet):
    exclude_child_manifests = filters.BooleanFilter(method='filter_child_manifests')

    sort = filters.OrderingFilter(
        fields=(
            ('pulp_created', 'created'),
        ),
    )

    class Meta:
        model = container_models.Manifest
        # Tag filters are supported, but are done in get_queryset. See the comment
        # there
        fields = {
            'digest': ['exact', 'icontains', 'contains', 'startswith'],
        }

    def filter_child_manifests(self, queryset, name, value):
        if value:
            return queryset.annotate(num_parents=Count("manifest_lists")).exclude(num_parents__gt=0)
        else:
            return queryset


class TagFilter(filterset.FilterSet):
    sort = filters.OrderingFilter(
        fields=(
            ("pulp_created", "pulp_created"),
            ("pulp_last_updated", "pulp_last_updated"),
            ("name", "name"),
        ),
    )

    class Meta:
        model = container_models.Tag
        # Tag filters are supported, but are done in get_queryset. See the comment
        # in ContainerRepositoryManifestViewSet.get_queryset
        fields = {
            "name": ["exact", "icontains", "contains", "startswith"],
            "tagged_manifest__digest": ["exact", "icontains", "contains", "startswith"],
        }


class HistoryFilter(filterset.FilterSet):
    sort = filters.OrderingFilter(
        fields=(
            ('pulp_created', 'created'),
            ('number', 'number'),
        ),
    )


class ContainerNamespaceViewSet(api_base.ModelViewSet):
    queryset = models.ContainerNamespace.objects.all()
    serializer_class = serializers.ContainerNamespaceDetailSerializer
    permission_classes = [access_policy.ContainerNamespaceAccessPolicy]
    lookup_field = "name"


class ContainerRepositoryViewSet(api_base.ModelViewSet):
    queryset = models.ContainerDistribution.objects.all().select_related('namespace')
    serializer_class = serializers.ContainerRepositorySerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = RepositoryFilter
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]
    lookup_field = "base_path"

    @extend_schema(
        description="Trigger an asynchronous delete task",
        responses={202: AsyncOperationResponseSerializer},
    )
    def destroy(self, request, *args, **kwargs):
        """
        Delete a distribution. If a push repository is associated to it, delete it as well.
        Perform orphan cleanup.
        """
        distribution = self.get_object()
        reservations = [distribution]
        ids_for_multi_delete = [
            (str(distribution.pk), "container", "ContainerDistributionSerializer"),
        ]
        if distribution.repository:
            reservations.append(distribution.repository)

            if distribution.repository.cast().PUSH_ENABLED:
                ids_for_multi_delete.append(
                    (str(distribution.repository.pk), "container",
                        "ContainerPushRepositorySerializer"),
                )
            else:
                ids_for_multi_delete.append(
                    (str(distribution.repository.pk), "container",
                        "ContainerRepositorySerializer"),
                )

            if distribution.repository.remote:
                reservations.append(distribution.repository.remote)
                ids_for_multi_delete.append(
                    (str(distribution.repository.remote.pk), "container",
                        "ContainerRemoteSerializer"),
                )

        # Delete the distribution, repository, and perform orphan cleanup
        async_result = dispatch(
            delete_container_distribution, reservations, args=(ids_for_multi_delete,)
        )

        return OperationPostponedResponse(async_result, request)


# provide some common methods across all <distro>/_content/ endpoints
class ContainerContentBaseViewset(api_base.ModelViewSet):
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]

    def get_distro(self):
        if getattr(self, "swagger_fake_view", False):
            # OpenAPI will generate a fake view with this attribute set
            return models.ContainerDistribution.objects.none()
        return get_object_or_404(
            models.ContainerDistribution, base_path=self.kwargs["base_path"])


class ContainerTagViewset(ContainerContentBaseViewset):
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]
    serializer_class = serializers.ContainerTagSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = TagFilter

    def get_queryset(self):
        if getattr(self, "swagger_fake_view", False):
            # OpenAPI will generate a fake view with this attribute set
            return container_models.Tag.objects.none()
        repo = self.get_distro().repository
        repo_version = repo.latest_version()
        return repo_version.get_content(container_models.Tag.objects)


class ContainerRepositoryManifestViewSet(ContainerContentBaseViewset):
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]
    filter_backends = (DjangoFilterBackend,)
    filterset_class = ManifestFilter

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return serializers.ContainerManifestDetailSerializer
        return serializers.ContainerManifestSerializer

    def get_queryset(self):
        if getattr(self, "swagger_fake_view", False):
            # OpenAPI will generate a fake view with this attribute set
            return container_models.Manifest.objects.none()
        repo = self.get_distro().repository
        repo_version = repo.latest_version()
        repo_content = repo_version.content.all()

        manifests = (
            repo_version.get_content(container_models.Manifest.objects)
            .prefetch_related(
                # Prefetch limits the tag list to tags in the current repo version.
                # Without it, tags from all repo versions are allowed which leads
                # to duplicates.
                Prefetch(
                    'tagged_manifests',
                    container_models.Tag.objects.filter(pk__in=repo_content)),
                Prefetch(
                    'blobs',
                    container_models.Blob.objects.prefetch_related(
                        Prefetch('_artifacts', to_attr='artifact_list')),
                    to_attr='blob_list'),
                'config_blob'
            )
        )

        # I know that this should go in the FilterSet, but I cannot for the life
        # of me figure out how to access base_path in the filterset. Without
        # base_path, it's impossible to filter the tags down to a specific repo version,
        # which means this query would end up pulling all of the tags in all repos
        # that match the tag param, which could potentially be thousands of objects.
        tag_filter = self.request.GET.get('tag', None)
        if tag_filter:
            manifests = manifests.filter(
                # tagged_manifests doesn't respect the Prefetch filtering, so
                # the repo version has to be filtered again here
                tagged_manifests__pk__in=repo_version.get_content(
                    container_models.Tag.objects).filter(name__icontains=tag_filter))

        return manifests

    def get_object(self):
        qs = self.get_queryset()
        # manifest_ref can be a tag name or a manifest digest
        manifest_ref = self.kwargs['manifest_ref']
        try:
            repo_version = self.get_distro().repository.latest_version()

            # we could just return tag.tagged_manifest, but using the tag in the
            # queryset allows this to take advantage of all the prefetching that
            # happens in get_queryset
            tag = repo_version.get_content(container_models.Tag.objects).get(name=manifest_ref)
            manifest = qs.get(tagged_manifests__pk=tag)
        except exceptions.ObjectDoesNotExist:
            manifest = get_object_or_404(qs, digest=manifest_ref)

        return manifest

    @extend_schema(
        description=(
            "Trigger an asynchronous task to remove a manifest and all its associated "
            "data by a digest"
        ),
        summary="Delete an image from a repository",
        responses={202: AsyncOperationResponseSerializer},
    )
    def destroy(self, request, *args, **kwargs):
        """Deletes a image manifest.

        - Looks up the image via a sha
        - Remove all tags that point to the selected manifest from the latest version of the repo.
        - Remove the selected image manifest from the selected repository using the pulp container
          remove_image function: This function will remove the manifest from the latest version
          of the repository and any blobs associated with the manifest that aren’t used by
          other manifests.
        - Call the reclaim disk space function on the selected repository, with the latest version
          of the repository preserved. This will clear out artifacts for content that isn’t in the
          latest version of the repository.
        """
        # Looks up the image via a sha
        manifest = self.get_object()
        repository = self.get_distro().repository
        latest_version = repository.latest_version()

        # Remove all tags that point to the selected manifest from the latest version of the repo.
        tags_pks = container_models.Tag.objects.filter(
            pk__in=latest_version.content.all(), tagged_manifest=manifest
        ).values_list("pk", flat=True)

        # Remove the selected image manifest from the selected repository using the pulp container
        content_unit_pks = [str(pk) for pk in list(tags_pks) + [manifest.pk]]

        # Call the recursive_remove_content from pulp_container + reclaim disk space
        async_result = dispatch(
            delete_container_image_manifest,
            [repository],
            args=(str(repository.pk), content_unit_pks, str(latest_version.pk))
        )

        return OperationPostponedResponse(async_result, request)


class ContainerRepositoryHistoryViewSet(ContainerContentBaseViewset):
    serializer_class = serializers.ContainerRepositoryHistorySerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = HistoryFilter

    def get_queryset(self):
        if getattr(self, "swagger_fake_view", False):
            # OpenAPI will generate a fake view with this attribute set
            return core_models.RepositoryVersion.objects.none()
        repo = self.get_distro().repository

        allowed_content_types = ['container.manifest', 'container.tag']

        # The ui only cares about repo versions where tags and manifests are added.
        # Pulp container revs the repo version each time any blobs are added, so
        # this filters out any repo versions where tags and manifests are unchanged.
        return (
            repo.versions.annotate(
                # Count the number of added/removed manifests and tags and use
                # the result to filter out any versions where tags and manifests
                # are unchanged.
                added_count=Count('added_memberships', filter=Q(
                    added_memberships__content__pulp_type__in=allowed_content_types)),
                removed_count=Count('removed_memberships', filter=Q(
                    removed_memberships__content__pulp_type__in=allowed_content_types))
            )
            .filter(Q(added_count__gt=0) | Q(removed_count__gt=0))
            .prefetch_related(
                Prefetch(
                    'added_memberships',
                    queryset=core_models.RepositoryContent.objects.filter(
                        content__pulp_type__in=allowed_content_types).select_related('content'),
                ),
                Prefetch(
                    'removed_memberships',
                    queryset=core_models.RepositoryContent.objects.filter(
                        content__pulp_type__in=allowed_content_types).select_related('content'),
                )
            ).order_by('-pulp_created')
        )


class ContainerReadmeViewSet(ContainerContentBaseViewset):
    queryset = models.ContainerDistroReadme.objects
    serializer_class = serializers.ContainerReadmeSerializer
    permission_classes = [access_policy.ContainerReadmeAccessPolicy]

    def get_object(self):
        distro = self.get_distro()
        return self.queryset.get_or_create(container=distro)[0]


class ContainerRegistryRemoteFilter(filterset.FilterSet):
    name = filters.CharFilter(field_name='name')
    url = filters.CharFilter(field_name='url')
    created_at = filters.CharFilter(field_name='pulp_created')
    updated_at = filters.CharFilter(field_name='pulp_last_updated')

    sort = filters.OrderingFilter(
        fields=(
            ('name', 'name'),
            ('url', 'url'),
            ('pulp_created', 'created_at'),
            ('pulp_last_updated', 'updated_at'),
        ),
    )

    class Meta:
        model = models.ContainerRegistryRemote
        fields = {
            'name': ['exact', 'icontains', 'contains', 'startswith'],
            'url': ['exact', 'icontains', 'contains', 'startswith'],
        }


class ContainerRegistryRemoteViewSet(api_base.ModelViewSet):
    queryset = models.ContainerRegistryRemote.objects.all()
    serializer_class = serializers.ContainerRegistryRemoteSerializer
    permission_classes = [access_policy.ContainerRegistryRemoteAccessPolicy]
    filterset_class = ContainerRegistryRemoteFilter


class ContainerRemoteViewSet(api_base.ModelViewSet):
    queryset = container_models.ContainerRemote.objects.all()
    serializer_class = serializers.ContainerRemoteSerializer
    permission_classes = [access_policy.ContainerRemoteAccessPolicy]



"""Tests that Collections can be uploaded to Pulp with the ansible-galaxy CLI."""

import subprocess
import tempfile
from unittest.case import skip

from pulp_smash.pulp3.bindings import delete_orphans
from pulp_smash.utils import http_get

from galaxy_ng.tests.functional.utils import TestCaseUsingBindings
from galaxy_ng.tests.functional.utils import set_up_module as setUpModule  # noqa:F401


class UploadCollectionTestCase(TestCaseUsingBindings):
    """Test whether ansible-galaxy can upload a Collection to Pulp."""

    def test_upload_collection(self):
        """Test whether ansible-galaxy can upload a Collection to Pulp."""
        delete_orphans()

        # Create namespace if it doesn't exist
        data = str(self.namespace_api.list().data)
        if "pulp" not in data:
            self.namespace_api.create(namespace={"name": "pulp", "groups": []})

        # Preapare ansible.cfg for ansible-galaxy CLI
        self.update_ansible_cfg("inbound-pulp")

        # In a temp dir, publish a collection using ansible-galaxy CLI
        with tempfile.TemporaryDirectory() as tmp_dir:
            content = http_get("https://galaxy.ansible.com/download/pulp-squeezer-0.0.9.tar.gz")
            collection_path = f"{tmp_dir}/pulp-squeezer-0.0.9.tar.gz"
            with open(collection_path, "wb") as f:
                f.write(content)

            cmd = "ansible-galaxy collection publish -vvv -c {}".format(collection_path)

            subprocess.run(cmd.split())

        # Verify that the collection was published
        collections = self.collections_api.list("published")
        self.assertEqual(collections.meta.count, 1)

        collection = self.collections_api.read(path="published", namespace="pulp", name="squeezer")
        self.assertEqual(collection.namespace, "pulp")
        self.assertEqual(collection.name, "squeezer")
        self.assertEqual(collection.highest_version["version"], "0.0.9")

        # Cleanup
        self.delete_collection(collection.namespace, collection.name)
        self.delete_namespace(collection.namespace)

    def test_uploaded_collection_logged(self):
        """Test whether a Collection uploaded via ansible-galaxy is logged correctly in API Access Log."""
        delete_orphans()

        # Create namespace if it doesn't exist
        data = str(self.namespace_api.list().data)
        if "pulp" not in data:
            self.namespace_api.create(namespace={"name": "pulp", "groups": []})

        # Preapare ansible.cfg for ansible-galaxy CLI
        self.update_ansible_cfg("inbound-pulp")

        # In a temp dir, publish a collection using ansible-galaxy CLI
        with tempfile.TemporaryDirectory() as tmp_dir:
            content = http_get("https://galaxy.ansible.com/download/pulp-squeezer-0.0.9.tar.gz")
            collection_path = f"{tmp_dir}/pulp-squeezer-0.0.9.tar.gz"
            with open(collection_path, "wb") as f:
                f.write(content)

            cmd = "ansible-galaxy collection publish -vvv -c {}".format(collection_path)

            subprocess.run(cmd.split())

            cmd = f"docker cp pulp:/var/log/galaxy_api_access.log {tmp_dir}"

            subprocess.run(cmd.split())

            with open(f"{tmp_dir}/galaxy_api_access.log") as f:
                log_contents = f.readlines()
                print(log_contents)

        # Verify that the collection was published
        collections = self.collections_api.list("published")
        self.assertEqual(collections.meta.count, 1)

        # Verify that the colletion publishing was logged in the api access log
        collection = self.collections_api.read(path="published", namespace="pulp", name="squeezer")
        for line in log_contents:
            if "INFO: Collection uploaded by user 'admin': " in line and 'pulp' in line:
                self.assertIn(collection.namespace, line)
                self.assertIn(collection.name, line)
                self.assertIn(collection.highest_version["version"], line)

        # Cleanup
        self.delete_collection(collection.namespace, collection.name)
        self.delete_namespace(collection.namespace)



from django.conf import settings
from django.contrib.auth.models import Group


def user_role(response, details, user=None, *args, **kwargs):
    """Pipeline used by SOCIAL AUTH to associate superuser priviledges."""
    if not user:
        return

    roles = response.get(settings.KEYCLOAK_ROLE_TOKEN_CLAIM, [])
    if not roles:
        return

    if not isinstance(roles, list):
        return

    is_admin = False
    if settings.KEYCLOAK_ADMIN_ROLE in roles:
        is_admin = True

    user.is_staff = is_admin
    user.is_admin = is_admin
    user.is_superuser = is_admin
    user.save()


def user_group(response, details, user=None, *args, **kwargs):
    """Pipeline used by social auth to update a users group associations."""
    if not user:
        return

    group_list = response.get(settings.KEYCLOAK_GROUP_TOKEN_CLAIM, [])

    if not isinstance(group_list, list):
        return

    groups = []
    for group_name in group_list:
        group_name_split = group_name.split("/")
        group_name = group_name_split[-1]
        new_group, created = Group.objects.get_or_create(name=group_name)
        groups.append(new_group)

    user.groups.clear()

    for group_item in groups:
        group_item.user_set.add(user)



from django.db import migrations


def add_content_guard(apps, schema_editor):
    """
    Enforce that all distributions have a content guard applied
    """
    AnsibleDistribution = apps.get_model('ansible', 'AnsibleDistribution')
    ContentRedirectContentGuard = apps.get_model('galaxy', 'ContentRedirectContentGuard')

    content_guard, _ = ContentRedirectContentGuard.objects.get_or_create(
        name='ContentRedirectContentGuard',
        pulp_type='ansible.ansible'
    )

    AnsibleDistribution.objects.filter(
        content_guard=None,
        pulp_type='ansible.ansible'
    ).update(
        content_guard=content_guard
    )


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0024_contentredirectcontentguard'),
    ]

    operations = [
        migrations.RunPython(
            code=add_content_guard,
            reverse_code=migrations.RunPython.noop
        )
    ]


import unittest

from urllib.parse import urlparse

from pulpcore.client.galaxy_ng.exceptions import ApiException
from pulp_container.tests.functional.api import rbac_base
from pulp_container.tests.functional.constants import DOCKERHUB_PULP_FIXTURE_1
from pulp_smash import cli
from pulp_smash.pulp3.bindings import monitor_task

from galaxy_ng.tests.functional.utils import TestCaseUsingBindings


class ContainerSyncandManifestListTestCase(TestCaseUsingBindings):
    """Test container sync and manifest lists.

    At the moment, the only way to get a container with a manifest list is to sync it from another
    registy. Because of that, this will test both here.

    Test manifest list + sync
    - set up docker registry
    - set up remote container in docker registry for pulp/test-fixture-1
        - set tags to: ml_i, manifest_a
    - perform sync
    - verify that the repo has two images with tags ml_i and manifest_a, that manifest_a is a manifest and that ml_i is a manifest list with two manifests in it

    Test multiple repositories in a registry
    - set up remote container in docker registry for pulp/test-fixture-1 with just tag manifest_a
    - set up remote container in docker registry for pulp/test-fixture-1 with just tag manifest_b
    - sync the entire registry
        - verify that both repos have synced
    """

    def setUp(self):
        self.docker_registry = self.container_registries_api.create({
            "name": "Docker Hub",
            "url": "https://registry.hub.docker.com",
        })
        self.addCleanup(self.smash_client.delete, f"{self.galaxy_api_prefix}/_ui/v1/execution-environments/registries/{self.docker_registry.pk}/")

    def _delete_remote_repo(self, remote):
        self.smash_client.delete(f"{self.galaxy_api_prefix}/_ui/v1/execution-environments/repositories/{remote.name}/")

    def test_manifests_and_remote_sync(self):
        remote_repo = self.container_remotes_api.create({
            "name": "test-repo1",
            "upstream_name": "pulp/test-fixture-1",
            "include_tags": ["ml_i", "manifest_b"],
            "registry": self.docker_registry.pk,
        })

        self.addCleanup(self._delete_remote_repo, remote_repo)

        # the galaxy_ng client doesn't seem to return anything with the sync function, so we're
        # using the api directly instead
        self.smash_client.post(f"{self.galaxy_api_prefix}/_ui/v1/execution-environments/repositories/{remote_repo.name}/_content/sync/")

        tags_list = self.container_repo_tags_api.list(remote_repo.name)
        self.assertEqual(tags_list.meta.count, 2)

        tags_list = self.container_repo_tags_api.list(remote_repo.name, name="manifest_b")
        self.assertEqual(tags_list.meta.count, 1)

        tagged_ml = tags_list.data[0]
        self.assertIn("manifest.v2", tagged_ml.tagged_manifest.media_type)

        # Note, the manifest_a tag in the pulp test repo points to a manifest that's part of the ml_i manifest list
        # so it won't show up in here since exclude_child_manifests filters out any manifest that is part of a manifest list.
        image_list = self.container_images_api.list(remote_repo.name, exclude_child_manifests=True)
        self.assertEqual(image_list.meta.count, 1)

        manifest_list = image_list.data[0]

        self.assertEqual(len(manifest_list.image_manifests), 2)
        for img in manifest_list.image_manifests:
            self.assertEqual(img.architecture, "amd64")

        self.assertIn("manifest.list.v2", manifest_list.media_type)


    def test_registry_sync(self):
        remote_repo1 = self.container_remotes_api.create({
            "name": "test-repo1",
            "upstream_name": "pulp/test-fixture-1",
            "include_tags": ["manifest_b"],
            "registry": self.docker_registry.pk,
        })

        remote_repo2 = self.container_remotes_api.create({
            "name": "test-repo2",
            "upstream_name": "pulp/test-fixture-1",
            "include_tags": ["manifest_b"],
            "registry": self.docker_registry.pk,
        })

        self.addCleanup(self._delete_remote_repo, remote_repo1)
        self.addCleanup(self._delete_remote_repo, remote_repo2)


        sync_task = self.smash_client.post(f"{self.galaxy_api_prefix}/_ui/v1/execution-environments/registries/{self.docker_registry.pk}/sync/")

        for task in sync_task['child_tasks']:
            monitor_task(task)

        tags_list = self.container_repo_tags_api.list(remote_repo1.name)
        self.assertEqual(tags_list.meta.count, 1)

        tags_list = self.container_repo_tags_api.list(remote_repo2.name)
        self.assertEqual(tags_list.meta.count, 1)



from pulpcore.plugin.tasking import add_and_remove, dispatch


def call_move_content_task(collection_version, source_repo, dest_repo):
    """Dispatches the move content task

    This is a wrapper to group copy_content and remove_content tasks
    because those 2 must run in sequence ensuring the same locks.

    """
    return dispatch(
        move_content,
        exclusive_resources=[source_repo, dest_repo],
        kwargs=dict(
            collection_version_pk=collection_version.pk,
            source_repo_pk=source_repo.pk,
            dest_repo_pk=dest_repo.pk,
        ),
    )


def move_content(collection_version_pk, source_repo_pk, dest_repo_pk):
    """Move collection version from one repository to another"""

    content = [collection_version_pk]

    # add content to the destination repo
    add_and_remove(
        dest_repo_pk,
        add_content_units=content,
        remove_content_units=[],
    )

    # remove content from source repo
    add_and_remove(
        source_repo_pk,
        add_content_units=[],
        remove_content_units=content,
    )



from prometheus_client import Counter


collection_import_attempts = Counter(
    "galaxy_api_collection_import_attempts",
    "count of collection import attempts"
)

collection_import_failures = Counter(
    "galaxy_api_collection_import_failures",
    "count of collection import failures"
)

collection_import_successes = Counter(
    "galaxy_api_collection_import_successes",
    "count of collections imported successfully"
)

collection_artifact_download_attempts = Counter(
    "galaxy_api_collection_artifact_download_attempts",
    "count of collection artifact download attempts"
)

collection_artifact_download_failures = Counter(
    "galaxy_api_collection_artifact_download_failures",
    "count of collection artifact download failures",
    ["status"]
)

collection_artifact_download_successes = Counter(
    "galaxy_api_collection_artifact_download_successes",
    "count of successful collection artifact downloads"
)



from django.utils.translation import gettext_lazy as _
from django.shortcuts import get_object_or_404
from drf_spectacular.utils import extend_schema_field
from rest_framework import serializers

from pulp_ansible.app import viewsets as pulp_viewsets
from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    CollectionRemote,
)

from galaxy_ng.app.constants import COMMUNITY_DOMAINS
from galaxy_ng.app.api import utils


class AnsibleDistributionSerializer(serializers.ModelSerializer):
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    class Meta:
        model = AnsibleDistribution
        fields = (
            'name',
            'base_path',
            'content_guard',
            'created_at',
            'updated_at',
        )


class AnsibleRepositorySerializer(serializers.ModelSerializer):
    distributions = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    last_sync_task = utils.RemoteSyncTaskField(source='remote')

    class Meta:
        model = AnsibleRepository
        fields = (
            'name',
            'description',
            'next_version',
            'distributions',
            'created_at',
            'updated_at',
            'last_sync_task',
        )

    @extend_schema_field(AnsibleDistributionSerializer(many=True))
    def get_distributions(self, obj):
        return [
            AnsibleDistributionSerializer(distro).data
            for distro in obj.distributions.all()
        ]


class CollectionRemoteSerializer(pulp_viewsets.CollectionRemoteSerializer):
    last_sync_task = utils.RemoteSyncTaskField(source='*')

    write_only_fields = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created', required=False)
    updated_at = serializers.DateTimeField(source='pulp_last_updated', required=False)

    proxy_password = serializers.CharField(
        help_text=_("Password for proxy authentication."),
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    proxy_username = serializers.CharField(
        help_text=_("User for proxy authentication."),
        allow_null=True,
        required=False,
        write_only=False,  # overwriting this as pulpcore defaults to True
    )
    token = serializers.CharField(
        allow_null=True,
        required=False,
        max_length=2000,
        write_only=True,
        style={'input_type': 'password'}
    )
    password = serializers.CharField(
        help_text=_("Remote password."),
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    username = serializers.CharField(
        help_text=_("Remote user."),
        allow_null=True,
        required=False,
        write_only=False,  # overwriting this as pulpcore defaults to True
    )
    name = serializers.CharField(read_only=True)
    repositories = serializers.SerializerMethodField()

    class Meta:
        model = CollectionRemote
        fields = (
            'pk',
            'name',
            'url',
            'auth_url',
            'token',
            'policy',
            'requirements_file',
            'created_at',
            'updated_at',
            'username',
            'password',
            'tls_validation',
            'client_key',
            'client_cert',
            'ca_cert',
            'last_sync_task',
            'repositories',
            'pulp_href',
            'download_concurrency',
            'proxy_url',
            'proxy_username',
            'proxy_password',
            'write_only_fields',
            'rate_limit'
        )
        extra_kwargs = {
            'name': {'read_only': True},
            'pulp_href': {'read_only': True},
            'client_key': {'write_only': True},
        }

    @extend_schema_field(serializers.ListField)
    def get_write_only_fields(self, obj):
        return utils.get_write_only_fields(self, obj)

    def validate(self, data):
        if not data.get('requirements_file') and any(
            [domain in data['url'] for domain in COMMUNITY_DOMAINS]
        ):
            raise serializers.ValidationError(
                detail={
                    'requirements_file':
                        _('Syncing content from community domains without specifying a '
                          'requirements file is not allowed.')
                }
            )

        init_data = self.initial_data
        if not data.get('proxy_password') and init_data.get('write_only_fields'):

            # filter proxy_password write_only_field
            proxy_pwd = next(
                item for item in init_data.get('write_only_fields')
                if item["name"] == "proxy_password"
            )
            repo = get_object_or_404(CollectionRemote, name=init_data.get('name'))

            # update proxy_password by value in db
            if proxy_pwd.get('is_set') and repo:
                data['proxy_password'] = repo.proxy_password

        return super().validate(data)

    @extend_schema_field(AnsibleRepositorySerializer(many=True))
    def get_repositories(self, obj):
        return [
            AnsibleRepositorySerializer(repo).data
            for repo in obj.repository_set.all()
        ]



# Generated by Django 2.2.15 on 2020-09-17 16:55

import contextlib

from django.db import migrations

def rename_default_repo(apps, schema_editor):
    AnsibleRepository = apps.get_model('ansible', 'AnsibleRepository')
    AnsibleDistribution = apps.get_model('ansible', 'AnsibleDistribution')
    db_alias = schema_editor.connection.alias

    old_name = 'automation-hub'
    new_name = 'published'

    if (
        AnsibleRepository.objects.using(db_alias).filter(name=new_name) or
        AnsibleDistribution.objects.using(db_alias).filter(name=new_name) or
        AnsibleDistribution.objects.using(db_alias).filter(base_path=new_name)
    ):
        return

    with contextlib.suppress(AnsibleRepository.DoesNotExist, AnsibleDistribution.DoesNotExist):
        repo = AnsibleRepository.objects.using(db_alias).get(name=old_name)
        distro = AnsibleDistribution.objects.using(db_alias).get(base_path=old_name)

        repo.name = new_name
        distro.name = new_name
        distro.base_path = new_name

        repo.save()
        distro.save()


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0007_merge_synclist'),
    ]

    operations = [
        migrations.RunPython(rename_default_repo, elidable=True),
    ]



import logging

from django.conf import settings
from django.contrib import auth as django_auth
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie
from rest_framework.authentication import SessionAuthentication

from rest_framework.response import Response
from rest_framework import status as http_code

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.access_control import access_policy

from galaxy_ng.app.api.ui.serializers import LoginSerializer

from requests import post as requests_post

log = logging.getLogger(__name__)

__all__ = (
    "LoginView",
    "LogoutView",
)


class _CSRFSessionAuthentication(SessionAuthentication):
    """CSRF-enforcing version of a SessionAuthentication class."""

    def authenticate(self, request):
        user = getattr(request._request, "user", None)
        self.enforce_csrf(request)
        return user, None


class LoginView(api_base.GenericAPIView):
    serializer_class = LoginSerializer
    authentication_classes = (_CSRFSessionAuthentication,)
    permission_classes = [access_policy.LoginAccessPolicy]

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        return Response(status=http_code.HTTP_204_NO_CONTENT)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=self.request.data)

        serializer.is_valid(raise_exception=True)

        username = serializer.validated_data['username']
        password = serializer.validated_data['password']

        user = django_auth.authenticate(request, username=username, password=password)
        if user is None:
            return Response(status=http_code.HTTP_403_FORBIDDEN)

        django_auth.login(request, user)
        return Response(status=http_code.HTTP_204_NO_CONTENT)


class LogoutView(api_base.APIView):
    permission_classes = [access_policy.LogoutAccessPolicy]

    def _oidc_logout(self, request):
        logout_url_str = "{keycloak}/auth/realms/{realm}/protocol/openid-connect/logout"
        logout_url = logout_url_str.format(keycloak=settings.KEYCLOAK_URL,
                                           realm=settings.KEYCLOAK_REALM)

        if not hasattr(request.user, 'social_auth'):
            return

        from social_django.models import UserSocialAuth
        try:
            social = request.user.social_auth.get(provider="keycloak")
        except UserSocialAuth.DoesNotExist:
            msg = "User does not have Social Auth object no openid-connect logout attemtped."
            log.warning(msg)
            return
        access_token = social.extra_data['access_token']
        refresh_token = social.extra_data['refresh_token']
        payload = {'client_id': settings.SOCIAL_AUTH_KEYCLOAK_KEY, 'refresh_token': refresh_token,
                   'client_secret': settings.SOCIAL_AUTH_KEYCLOAK_SECRET}
        headers = {"Authorization": "Bearer {access_token}".format(access_token=access_token)}
        response = requests_post(
            url=logout_url,
            headers=headers,
            data=payload,
            verify=settings.GALAXY_VERIFY_KEYCLOAK_SSL_CERTS
        )

        if response.status_code == http_code.HTTP_200_OK:
            log.debug("Logout of openid-connect client successful.")
        else:
            msg = "Log out for openid-connect failed with status {status} and content: {content}"
            log.warning(msg.format(status=response.status_code, content=response.text))

    def post(self, request, *args, **kwargs):
        # Trigger logout in Keycloak
        if settings.get("SOCIAL_AUTH_KEYCLOAK_KEY"):
            self._oidc_logout(request)

        django_auth.logout(request)
        return Response(status=http_code.HTTP_204_NO_CONTENT)



import logging

from pulp_ansible.app.models import Collection, CollectionVersion
from pulp_container.app import tasks as pulp_container_tasks
from pulpcore.app.tasks import orphan_cleanup, reclaim_space
from pulpcore.plugin.tasking import add_and_remove

log = logging.getLogger(__name__)


def _cleanup_old_versions(repo):
    """Delete all the old versions of the given repository."""
    for version in repo.versions.complete().order_by("-number")[1:]:
        version.delete()


def _remove_collection_version_from_repos(collection_version):
    """Remove CollectionVersion from each repo, and ensure no old RepositoryVersion."""
    for repo in collection_version.repositories.all():
        # enforce retain_repo_versions is set to 1 on the repository
        if repo.retain_repo_versions != 1:
            repo.retain_repo_versions = 1
            repo.save()

        # remove CollectionVersion from latest RepositoryVersion
        add_and_remove(repo.pk, add_content_units=[], remove_content_units=[collection_version.pk])
        _cleanup_old_versions(repo)
    collection_version.save()


def delete_collection_version(collection_version_pk):
    """Task to delete CollectionVersion object.

    Sequentially do the following in a single task:
    1. Call _remove_collection_version_from_repos
    2. Run orphan_cleanup to delete the CollectionVersion
    3. Delete Collection if it has no more CollectionVersion
    """

    collection_version = CollectionVersion.objects.get(pk=collection_version_pk)
    collection = collection_version.collection

    _remove_collection_version_from_repos(collection_version)

    log.info("Running orphan_cleanup to delete CollectionVersion object and artifact")
    orphan_cleanup(content_pks=[collection_version.pk], orphan_protection_time=0)

    if not collection.versions.all():
        log.info("Collection has no more versions, deleting collection {}".format(collection))
        collection.delete()


def delete_collection(collection_pk):
    """Task to delete Collection object.

    Sequentially do the following in a single task:
    1. For each CollectionVersion call _remove_collection_version_from_repos
    2. Run orphan_cleanup to delete the CollectionVersions
    3. Delete Collection
    """

    collection = Collection.objects.get(pk=collection_pk)
    version_pks = []
    for version in collection.versions.all():
        _remove_collection_version_from_repos(version)
        version_pks.append(version.pk)

    log.info("Running orphan_cleanup to delete CollectionVersion objects and artifacts")
    orphan_cleanup(content_pks=version_pks, orphan_protection_time=0)

    log.info("Deleting collection {}".format(collection))
    collection.delete()


def delete_container_distribution(instance_ids):
    """Deletes a container distribution and push repository related."""

    log.info("Running container.general_multi_delete to delete distro and repo")
    pulp_container_tasks.general_multi_delete(instance_ids=instance_ids)

    log.info("Running orphan_cleanup to delete Container objects and artifacts")
    orphan_cleanup(content_pks=None, orphan_protection_time=10)


def delete_container_image_manifest(repository_pk, content_unit_pks, repo_latest_version_pk):
    """Deletes a container image manifest."""

    log.info(f"Running delete manifest for {repository_pk}")
    pulp_container_tasks.recursive_remove_content(
        repository_pk=repository_pk,
        content_units=content_unit_pks,
    )

    log.info(f"Reclaiming disk space for {repository_pk}")
    reclaim_space(repo_pks=[repository_pk], keeplist_rv_pks=[repo_latest_version_pk], force=True)



from django.db import models
from django.urls import reverse
from django_lifecycle import LifecycleModel

from pulpcore.plugin.models import (
    AutoDeleteObjPermsMixin,
)
from pulp_ansible.app.models import CollectionImport as PulpCollectionImport
from .namespace import Namespace


__all__ = (
    "CollectionImport",
)


class CollectionImport(LifecycleModel, AutoDeleteObjPermsMixin):
    """
    A model representing a mapping between pulp task id and task parameters.

    Fields:
        task_id: Task UUID.
        created_at: Task creation date time.
        name: Collection name.
        version: Collection version.

    Relations:
        namespace: Reference to a namespace.
    """
    # task_id = models.UUIDField(primary_key=True)
    task_id = models.OneToOneField(PulpCollectionImport,
                                   primary_key=True,
                                   on_delete=models.CASCADE,
                                   db_column='task_id',
                                   related_name='galaxy_import')
    # pulp_task = models.ForeignKey(Task, on_delete=models.CASCADE, default=task_id)

    created_at = models.DateTimeField()

    namespace = models.ForeignKey(Namespace, on_delete=models.CASCADE)
    name = models.CharField(max_length=64, editable=False)
    version = models.CharField(max_length=32, editable=False)

    class Meta:
        ordering = ['-task_id']

    def get_absolute_url(self):
        return reverse('galaxy:api:content:collection-import', args=[str(self.task_id)])



import logging

from rest_framework import status as http_code

from pulp_ansible.app import models as pulp_ansible_models

from galaxy_ng.app.constants import DeploymentMode
from galaxy_ng.app import models as galaxy_models
from galaxy_ng.app.models import auth as auth_models

from .base import BaseTestCase, get_current_ui_url

log = logging.getLogger(__name__)
logging.getLogger().setLevel(logging.DEBUG)


class TestUIDistributions(BaseTestCase):
    default_owner_permissions = [
        'change_synclist',
        'view_synclist',
        'delete_synclist'
    ]

    def setUp(self):
        super().setUp()
        self.user = auth_models.User.objects.create(username='admin')
        self.distro_url = get_current_ui_url('distributions-list')
        self.my_distro_url = get_current_ui_url('my-distributions-list')

        self.group = auth_models.Group.objects.create(name='test1_group')
        self.user.groups.add(self.group)

        self.synclist_repo = self._create_repository('123-synclist')
        self.repo2 = self._create_repository('other-repo')
        self.inbound_repo = self._create_repository('inbound-test')

        self.synclist_distro = self._create_distribution(self.synclist_repo)
        self.repo2_distro = self._create_distribution(self.repo2)
        self.inbound_distro = self._create_distribution(self.inbound_repo)

        upstream_repo = self._create_repository('upstream')
        self.synclist = self._create_synclist(
            '123-synclist',
            self.synclist_repo,
            upstream_repo,
            groups=[self.group]
        )

    def _create_distribution(self, repo):
        return pulp_ansible_models.AnsibleDistribution.objects.create(
            repository=repo, name=repo.name, base_path=repo.name)

    def _create_repository(self, name):
        repo = pulp_ansible_models.AnsibleRepository.objects.create(name=name)
        return repo

    def _create_synclist(
        self, name, repository, upstream_repository, collections=None, namespaces=None,
        policy=None, groups=None,
    ):
        synclist = galaxy_models.SyncList.objects.create(
            name=name, repository=repository, upstream_repository=upstream_repository)
        if groups:
            groups_to_add = {}
            for group in groups:
                groups_to_add[group] = self.default_owner_permissions
            synclist.groups = groups_to_add
        return synclist

    def test_distribution_list(self):
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            self.client.force_authenticate(user=self.user)
            data = self.client.get(self.distro_url).data

            self.assertEqual(len(data['data']), 6)
            # 5 distros created by data migration and one `other-repo` created
            # on test setUp.

        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.INSIGHTS.value):
            self.client.force_authenticate(user=self.user)
            response = self.client.get(self.distro_url)
            self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)

    def test_my_distribution_list(self):
        self.client.force_authenticate(user=self.user)
        data = self.client.get(self.my_distro_url).data
        self.assertEqual(len(data['data']), 1)
        self.assertEqual(data['data'][0]['pulp_id'], str(self.synclist_distro.pk))
        self.assertEqual(data['data'][0]['repository']['name'], self.synclist_repo.name)



from django.db import models

from pulpcore.plugin.models import AutoDeleteObjPermsMixin
from pulp_ansible.app.models import AnsibleRepository, Collection
from django_lifecycle import LifecycleModel

from galaxy_ng.app.access_control.mixins import GroupModelPermissionsMixin

from . import namespace as namespace_models


class SyncList(
    LifecycleModel, GroupModelPermissionsMixin, AutoDeleteObjPermsMixin
):

    POLICY_CHOICES = [
        ("exclude", "exclude"),
        ("include", "include"),
    ]

    name = models.CharField(max_length=64, unique=True, blank=False)
    policy = models.CharField(max_length=64, choices=POLICY_CHOICES, default="exclude")

    upstream_repository = models.ForeignKey(
        AnsibleRepository, on_delete=models.CASCADE, related_name="upstream_repositories"
    )
    repository = models.ForeignKey(
        AnsibleRepository, on_delete=models.CASCADE, related_name="repositories"
    )
    collections = models.ManyToManyField(Collection)
    namespaces = models.ManyToManyField(namespace_models.Namespace)



from io import StringIO
from django.contrib.auth import get_user_model
from django.core.management import call_command, CommandError
from django.test import TestCase

User = get_user_model()


class TestCreateUserCommand(TestCase):
    def setUp(self):
        super().setUp()

    def test_command_output(self):
        with self.assertRaisesMessage(
            CommandError, 'Error: the following arguments are required: --user'
        ):
            call_command('create-user')

    def test_add_user(self):
        out = StringIO()
        call_command('create-user', '--user', 'system:foo', stdout=out)
        self.assertIn("Created user 'system:foo'", out.getvalue())

    def test_user_already_exists(self):
        out = StringIO()
        call_command('create-user', '--user', 'system:foo', stdout=out)
        call_command('create-user', '--user', 'system:foo', stdout=out)
        self.assertIn("User 'system:foo' already exists", out.getvalue())



# Generated by Django 2.2.15 on 2020-08-24 16:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0004_rbac'),
        ('galaxy', '0005_merge_synclist_inbound'),
    ]

    operations = [
    ]



from io import StringIO
from django.core.management import call_command, CommandError
from django.test import TestCase
from galaxy_ng.app.models.auth import Group


class TestAssignPermissionCommand(TestCase):
    def setUp(self):
        super().setUp()
        group = Group.objects.create(name='administrator')
        group.save()

    def test_command_output(self):
        with self.assertRaisesMessage(
            CommandError,
            'Error: the following arguments are required: group, permissions'
        ):
            call_command('assign-permission')

    def test_add_permission(self):
        out = StringIO()
        call_command(
            'assign-permission', 'administrator', 'galaxy.add_namespace',
            stdout=out)
        self.assertIn(
            "Assigned requested permission to group 'administrator'",
            out.getvalue())
        admin_group = Group.objects.get(name='administrator')
        self.assertIn(
            'add_namespace',
            [perm.codename for perm in admin_group.permissions.all()]
        )

    def test_add_multiple_permissions(self):
        out = StringIO()
        call_command('assign-permission', 'administrator',
                     'galaxy.add_namespace', 'galaxy.change_namespace', stdout=out)
        self.assertIn("Assigned requested permission to group 'administrator'", out.getvalue())
        admin_group = Group.objects.get(name='administrator')
        self.assertIn(
            'add_namespace',
            [perm.codename for perm in admin_group.permissions.all()]
        )

    def test_group_not_found(self):
        with self.assertRaisesMessage(
            CommandError,
            'Group system:foo does not exist. Please provide a valid group '
            'name'
        ):
            call_command('assign-permission', 'system:foo',
                         'galaxy.add_namespace')

    def test_permission_not_found(self):
        out = StringIO()
        with self.assertRaisesMessage(
            CommandError,
                "Permission galaxy.foo not found. Please provide a valid "
                "permission in the form 'app_label.codename'"
        ):
            call_command('assign-permission', 'administrator', 'galaxy.foo',
                         stdout=out)

    def test_permission_format(self):
        out = StringIO()
        with self.assertRaisesMessage(
            CommandError,
                "Invalid permission format for foo. Expecting "
                "'app_label.codename'"
        ):
            call_command('assign-permission', 'administrator', 'foo',
                         stdout=out)



import hashlib
import os
import re
from url_normalize import url_normalize
from urllib.parse import urlparse

from django.db import models

from pulpcore.plugin.models import ContentGuard


def _gen_secret():
    return os.urandom(32)


class ContentRedirectContentGuard(ContentGuard):
    """
    Content guard to allow preauthenticated redirects to the content app.
    """

    TYPE = "content_redirect"

    shared_secret = models.BinaryField(max_length=32, default=_gen_secret)

    def permit(self, request):
        """
        Permit preauthenticated redirects from pulp-api.
        """
        try:
            signed_url = request.url
            validate_token = request.query["validate_token"]
            hex_salt, hex_digest = validate_token.split(":", 1)
            salt = bytes.fromhex(hex_salt)
            digest = bytes.fromhex(hex_digest)
            url = re.sub(r"\?validate_token=.*$", "", str(signed_url))
            if not digest == self._get_digest(salt, url):
                raise PermissionError("Access not authenticated")
        except (KeyError, ValueError):
            raise PermissionError("Access not authenticated")

    def preauthenticate_url(self, url, salt=None):
        """
        Add validate_token to urls query string.
        """
        if not salt:
            salt = _gen_secret()
        hex_salt = salt.hex()
        digest = self._get_digest(salt, url).hex()
        url = url + f"?validate_token={hex_salt}:{digest}"
        return url

    def _get_digest(self, salt, url):
        url_parts = urlparse(url_normalize(url))
        hasher = hashlib.sha256()
        hasher.update(salt)
        hasher.update(url_parts.path.encode())
        hasher.update(b"?")
        hasher.update(url_parts.query.encode())
        hasher.update(self.shared_secret)
        return hasher.digest()

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"



import logging
from unittest.case import skip
from uuid import uuid4

from django.test.utils import override_settings
from django.urls.base import reverse
from orionutils.generator import build_collection
from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    Collection,
    CollectionVersion,
)
from rest_framework import status

from galaxy_ng.app import models
from galaxy_ng.app.api.v3.viewsets.collection import get_collection_dependents, get_dependents
from galaxy_ng.app.constants import DeploymentMode
from galaxy_ng.tests.constants import TEST_COLLECTION_CONFIGS

from .base import BaseTestCase

log = logging.getLogger(__name__)


def _create_repo(name, **kwargs):
    repo = AnsibleRepository.objects.create(name=name, **kwargs)
    AnsibleDistribution.objects.create(
        name=name, base_path=name, repository=repo
    )
    return repo


def _get_create_version_in_repo(namespace, collection, repo, **kwargs):
    collection_version, _ = CollectionVersion.objects.get_or_create(
        namespace=namespace,
        name=collection.name,
        collection=collection,
        **kwargs,
    )
    qs = CollectionVersion.objects.filter(pk=collection_version.pk)
    with repo.new_version() as new_version:
        new_version.add_content(qs)
    return collection_version


@override_settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value)
class TestCollectionViewsets(BaseTestCase):

    def setUp(self):
        super().setUp()

        self.admin_user = self._create_user("admin")
        self.pe_group = self._create_partner_engineer_group()
        self.admin_user.groups.add(self.pe_group)
        self.admin_user.save()

        self.namespace = models.Namespace.objects.create(name='col_namespace')
        self.collection = Collection.objects.create(
            namespace=self.namespace, name='col_collection'
        )
        self.repo = _create_repo(name='col_repo')

        self.version_1_1_1 = _get_create_version_in_repo(
            self.namespace,
            self.collection,
            self.repo,
            version="1.1.1",
        )
        _get_create_version_in_repo(
            self.namespace,
            self.collection,
            self.repo,
            version="1.1.2",
        )

        # TODO: Upload pulp_ansible/tests/assets collection
        #       or create dummy ContentArtifacts directly

        self.collections_url = reverse(
            'galaxy:api:content:v3:collections-list',
            kwargs={
                'path': self.repo.name,
            }
        )

        self.collections_detail_url = reverse(
            'galaxy:api:content:v3:collections-detail',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name
            }
        )

        self.versions_url = reverse(
            'galaxy:api:content:v3:collection-versions-list',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name
            }
        )

        self.versions_detail_url = reverse(
            'galaxy:api:content:v3:collection-versions-detail',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name,
                'version': '1.1.2'
            }
        )

        self.collection_upload_url = reverse(
            "galaxy:api:v3:default-content:collection-artifact-upload"
        )

        # The following tests use endpoints related to
        # issue https://issues.redhat.com/browse/AAH-224
        # For now endpoints are temporary deactivated
        # self.all_collections_url = reverse(
        #     "galaxy:api:content:v3:all-collections-list",
        #     kwargs={
        #         "path": self.repo.name,
        #     },
        # )
        #
        # self.all_versions_url = reverse(
        #     "galaxy:api:content:v3:all-collection-versions-list",
        #     kwargs={
        #         "path": self.repo.name,
        #     },
        # )
        #
        # self.metadata_url = reverse(
        #     "galaxy:api:content:v3:repo-metadata",
        #     kwargs={
        #         "path": self.repo.name,
        #     },
        # )

        # used for href tests
        self.pulp_href_fragment = "pulp_ansible/galaxy"

    def upload_collections(self, namespace=None, collection_configs=None):
        """using the config from TEST_COLLECTION_CONFIGS,
        generates and uploads collections to pulp_ansible/galaxy.
        """
        collection_configs = collection_configs or TEST_COLLECTION_CONFIGS
        self._create_namespace(namespace, groups=[self.pe_group])
        collections = []
        for config in collection_configs:
            config["namespace"] = namespace
            collection = build_collection("skeleton", config=config)
            response = self.client.post(
                self.collection_upload_url, {"file": open(collection.filename, "rb")}
            )
            collections.append((collection, response))
        return collections

    def test_upload_collection(self):
        """Test successful upload of collections generated with orionutils.

        NOTE: This test is testing only the upload view of the collection
              After the collection is uploaded a pulp task is created to import it.
              but there are no workers running, so the import task is not executed.

        TODO: Call the move/promote of task manually or find a way to execute a eager worker.
              (or even better, move this tests to functional testing)
        """

        self.client.force_authenticate(user=self.admin_user)
        collections = self.upload_collections(namespace=uuid4().hex)
        self.assertEqual(len(collections), 12)
        # assert each collection returned a 202
        for collection in collections:
            self.assertEqual(collection[1].status_code, 202)
        # Upload is performed but the collection is not yet imported

    def test_collections_list(self):
        """Assert the call to v3/collections returns correct
        collections and versions
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.collections_url)
        self.assertEqual(response.data['meta']['count'], 1)
        self.assertEqual(response.data['data'][0]['deprecated'], False)
        self.assertEqual(
            response.data['data'][0]['highest_version']['version'], '1.1.2'
        )

        # Ensure href is overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["versions_url"])
        self.assertNotIn(
            self.pulp_href_fragment, response.data["data"][0]["highest_version"]["href"]
        )

    @skip("https://issues.redhat.com/browse/AAH-224")
    def test_unpaginated_collections_list(self):
        """Assert the call to v3/collections/all returns correct
        collections and versions
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.all_collections_url)
        self.assertEqual(response.data[0]['deprecated'], False)
        self.assertEqual(response.data[0]['highest_version']['version'], '1.1.2')

        # Ensure href is overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["versions_url"])
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["highest_version"]["href"])

    def test_collections_detail(self):
        """Assert detail view for v3/collections/namespace/name
        retrurns the same values
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.collections_detail_url)
        self.assertEqual(response.data['namespace'], self.namespace.name)
        self.assertEqual(response.data['deprecated'], False)
        self.assertEqual(
            response.data['highest_version']['version'], '1.1.2'
        )

        # Ensure hrefs are overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data["versions_url"])
        self.assertNotIn(self.pulp_href_fragment, response.data["highest_version"]["href"])

        # Check response after DELETE
        response = self.client.delete(self.collections_detail_url)
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)
        self.assertIn("task", response.data.keys())

    def test_collection_version_delete_dependency_check_positive_match(self):
        baz_collection = Collection.objects.create(namespace=self.namespace, name="baz")
        for counter, dep_version in enumerate(["1.1.1", ">=1", "<2", "~1", "*"]):
            baz_version = _get_create_version_in_repo(
                self.namespace,
                baz_collection,
                self.repo,
                version=counter,
                dependencies={f"{self.namespace.name}.{self.collection.name}": dep_version},
            )
            self.assertIn(baz_version, get_dependents(self.version_1_1_1))

    def test_collection_version_delete_dependency_check_negative_match(self):
        baz_collection = Collection.objects.create(namespace=self.namespace, name="baz")
        for counter, dep_version in enumerate(["1.1.2", ">1", "<1.1.1", "~=2"]):
            baz_version = _get_create_version_in_repo(
                self.namespace,
                baz_collection,
                self.repo,
                version=counter,
                dependencies={f"{self.namespace.name}.{self.collection.name}": dep_version},
            )
            self.assertNotIn(baz_version, get_dependents(self.version_1_1_1))

    def test_collection_delete_dependency_check(self):
        baz_collection = Collection.objects.create(namespace=self.namespace, name="baz")
        for counter, dep_version in enumerate(["1.1.1", ">=1", "<2", "~1", "*"]):
            baz_version = _get_create_version_in_repo(
                self.namespace,
                baz_collection,
                self.repo,
                version=counter,
                dependencies={f"{self.namespace.name}.{self.collection.name}": dep_version},
            )
            self.assertIn(baz_version, get_collection_dependents(self.collection))
        self.assertFalse(get_collection_dependents(baz_collection))

    def test_collection_versions_list(self):
        """Assert v3/collections/namespace/name/versions/
        lists all available versions.

        Assert that each version href returns correct data and
        artifacts.
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.versions_url)
        self.assertEqual(response.data['meta']['count'], 2)
        self.assertEqual(response.data['data'][0]['version'], '1.1.2')
        self.assertEqual(response.data['data'][1]['version'], '1.1.1')

        # Ensure hrefs are overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["href"])

        # TODO: implement subtests for each version after the
        # upload of artifacts has been implemented in `self.setUp`
        # for version in response.data['data']:
        #     with self.subTest(version=version['version):
        #         vresponse = self.client.get(version['href'])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["href"])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["collection"]["href"])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["download_url"])

    # def test_unpaginated_collection_versions_list(self):
    #     """Assert the call to v3/collections/all returns correct
    #     collections and versions
    #     """
    #     self.client.force_authenticate(user=self.admin_user)
    #     response = self.client.get(self.all_versions_url)
    #     self.assertEqual(response.data[0]['version'], '1.1.2')

    #     # Ensure href is overwritten
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["href"])
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["collection"]["href"])
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["download_url"])

    #     # Ensure some fields are in
    #     for field in ('metadata', 'namespace', 'name', 'artifact'):
    #         with self.subTest(field=field):
    #             self.assertIn(field, response.data[0])

    #     # Ensure some fields are not in
    #     for field in ('manifest', 'files'):
    #         with self.subTest(field=field):
    #             self.assertNotIn(field, response.data[0])



import logging

from galaxy_ng.app.constants import DeploymentMode
from galaxy_ng.app.models import auth as auth_models

from .base import BaseTestCase, get_current_ui_url

log = logging.getLogger(__name__)
logging.getLogger().setLevel(logging.DEBUG)


class TestLocalization(BaseTestCase):

    def setUp(self):
        super().setUp()
        self.group_url = get_current_ui_url('groups')
        self.admin_user = auth_models.User.objects.create(username='admin',
                                                          is_superuser=True)
        self.group = auth_models.Group.objects.create(name='test_group')
        self.user.groups.add(self.group)
        self.admin_user.save()

    def _switch_lang_post(self, language):
        response = self.client.post(
            self.group_url,
            {"name": "test_group"},
            HTTP_ACCEPT_LANGUAGE=language
        )
        return response.data['errors'][0]['detail']

    def _switch_lang_get(self, language):
        response = self.client.get(
            self.group_url + 'notfoundgroup/',
            HTTP_ACCEPT_LANGUAGE=language
        )
        return response.data['errors'][0]['title']

    def test_auto_localization(self):
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            self.client.force_authenticate(user=self.admin_user)

            response = self.client.get(
                self.group_url + 'notfoundgroup/'
            )
            self.assertEqual(
                response.data['errors'][0]['title'],
                'Not found.'
            )

            response = self._switch_lang_get('es')
            self.assertEqual(
                response,
                'No encontrado.'
            )

            response = self._switch_lang_get('fr')
            self.assertEqual(
                response,
                'Pas trouvé.'
            )

            response = self._switch_lang_get('nl')
            self.assertEqual(
                response,
                'Niet gevonden.'
            )

            response = self._switch_lang_get('ja')
            self.assertEqual(
                response,
                '見つかりませんでした。'
            )

            response = self._switch_lang_get('zh')
            self.assertEqual(
                response,
                '未找到。'
            )

            response = self._switch_lang_get('cs')
            self.assertEqual(
                response,
                'Not found.'
            )

    def test_localization_files(self):
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            self.client.force_authenticate(user=self.admin_user)

            response = self.client.post(
                self.group_url,
                {"name": "test_group"}
            )
            self.assertEqual(
                response.data['errors'][0]['detail'],
                'A group named test_group already exists.'
            )

            response = self._switch_lang_post('fr')
            self.assertEqual(
                response,
                'Un groupe nommé test_group existe déjà.'
            )

            response = self._switch_lang_post('es')
            self.assertEqual(
                response,
                'Ya existe un grupo llamado test_group.'
            )

            response = self._switch_lang_post('nl')
            self.assertEqual(
                response,
                'Een groep met de naam test_group bestaat al.'
            )

            response = self._switch_lang_post('ja')
            self.assertEqual(
                response,
                'test_group という名前のグループはすでに存在します。'
            )

            response = self._switch_lang_post('zh')
            self.assertEqual(
                response,
                '名为 test_group 的组已存在。'
            )

            response = self._switch_lang_post('cs')
            self.assertEqual(
                response,
                'A group named test_group already exists.'
            )
